#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of Pymetrick.  The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.

"""Modulo para generar paginas html5"""

try:
    from pymetrick.version import __author__, __version__, __license__, __copyright__
except ImportError:
    raise ImportError(
        'Failed to find (autogenerated) version.py '
    )

__date__ = '2012-09-21'
__credits__ = ''
__text__ = 'Generador de codigo HTML5 a partir de ficheros'
__file__ = 'template.py'

#--- CHANGES ------------------------------------------------------------------
# 2012-09-21 v0.01 PL: - First version
# 2017-10-02 v0.43 PL: - Migrar a PYTHON 3.6

import sys
# sin bytecode
sys.dont_write_bytecode = True
import os
import string
import pdb
import re
import logging
import logging.handlers
try:
    import simplejson as json
except ImportError:
    import json
from pymetrick.common import _weekdayname,_monthname

long = int

# Comprimir HTML
HTML_COMPRESS = False

acceptable_head_elements = ['head','title','base','link','meta','style','script']

acceptable_head_attributes = ['name','content','type','size','href']

acceptable_body_elements = ['a', 'abbr', 'acronym', 'address', 'area',
        'article', 'aside', 'audio', 'b', 'big', 'blockquote', 'br', 'button',
        'canvas', 'caption', 'center', 'cite', 'code', 'col', 'colgroup',
        'command', 'datagrid', 'datalist', 'dd', 'del', 'details', 'dfn',
        'dialog', 'dir', 'div', 'dl', 'dt', 'em', 'event-source', 'fieldset',
        'figcaption', 'figure', 'footer', 'font', 'form', 'header', 'hgroup', 'h1',
        'h2', 'h3', 'h4', 'h5', 'h6', 'hr', 'i', 'img', 'input', 'ins',
        'keygen', 'kbd', 'label', 'legend', 'li', 'm', 'map', 'mark', 'menu', 'meter',
        'multicol', 'nav', 'nextid', 'ol', 'output', 'optgroup', 'option',
        'p', 'pre', 'progress', 'q', 's', 'samp', 'section', 'select',
        'small', 'sound', 'source', 'spacer', 'span', 'strike', 'strong',
        'sub', 'sup', 'table', 'tbody', 'td', 'textarea', 'time', 'tfoot',
        'th', 'thead', 'tr', 'tt', 'u', 'ul', 'var', 'video']

self_closing_tag = ['base','br','col','hr','img','input','link','meta','param']

acceptable_attributes = ['abbr', 'accept', 'accept-charset', 'accesskey',
        'action', 'align', 'alt', 'autocomplete', 'autofocus', 'axis',
        'background', 'balance', 'bgcolor', 'bgproperties', 'border',
        'bordercolor', 'bordercolordark', 'bordercolorlight', 'bottompadding',
        'cellpadding', 'cellspacing', 'ch', 'challenge', 'char', 'charoff',
        'choff', 'charset', 'checked', 'cite', 'class', 'clear', 'color',
        'cols', 'colspan', 'compact', 'contenteditable', 'controls', 'coords',
        'data', 'datafld', 'datapagesize', 'datasrc', 'datetime', 'default',
        'delay', 'dir', 'disabled', 'draggable', 'dynsrc', 'enctype', 'end',
        'face', 'for', 'form', 'frame', 'galleryimg', 'gutter', 'headers',
        'height', 'hidefocus', 'hidden', 'high', 'href', 'hreflang', 'hspace',
        'icon', 'id', 'inputmode', 'ismap', 'keytype', 'label', 'leftspacing',
        'lang', 'list', 'longdesc', 'loop', 'loopcount', 'loopend',
        'loopstart', 'low', 'lowsrc', 'max', 'maxlength', 'media', 'method',
        'min', 'multiple', 'name', 'nohref', 'noshade', 'nowrap', 'open',
        'optimum', 'pattern', 'ping', 'point-size', 'prompt', 'pqg',
        'radiogroup', 'readonly', 'rel', 'repeat-max', 'repeat-min',
        'replace', 'required', 'rev', 'rightspacing', 'rows', 'rowspan',
        'rules', 'scope', 'selected', 'shape', 'size', 'span', 'src', 'start',
        'step', 'style', 'summary', 'suppress', 'tabindex', 'target',
        'template', 'title', 'toppadding', 'type', 'unselectable', 'usemap',
        'urn', 'valign', 'value', 'variable', 'volume', 'vspace', 'vrml',
        'width', 'wrap', 'xml:lang']

acceptable_css_properties = ['azimuth', 'background-color',
        'border-bottom-color', 'border-collapse', 'border-color',
        'border-left-color', 'border-right-color', 'border-top-color', 'clear',
        'color', 'cursor', 'direction', 'display', 'elevation', 'float', 'font',
        'font-family', 'font-size', 'font-style', 'font-variant', 'font-weight',
        'height', 'letter-spacing', 'line-height', 'overflow', 'pause',
        'pause-after', 'pause-before', 'pitch', 'pitch-range', 'richness',
        'speak', 'speak-header', 'speak-numeral', 'speak-punctuation',
        'speech-rate', 'stress', 'text-align', 'text-decoration', 'text-indent',
        'unicode-bidi', 'vertical-align', 'voice-family', 'volume',
        'white-space', 'width']

acceptable_css_keywords = ['auto', 'aqua', 'black', 'block', 'blue',
        'bold', 'both', 'bottom', 'brown', 'center', 'collapse', 'dashed',
        'dotted', 'fuchsia', 'gray', 'green', '!important', 'italic', 'left',
        'lime', 'maroon', 'medium', 'none', 'navy', 'normal', 'nowrap', 'olive',
        'pointer', 'purple', 'red', 'right', 'solid', 'silver', 'teal', 'top',
        'transparent', 'underline', 'white', 'yellow']



def format_date_time(timestamp):
    year, month, day, hh, mm, ss, wd, y, z = time.gmtime(timestamp)
    return "%s, %02d %3s %4d %02d:%02d:%02d GMT" % (_weekdayname[wd], day, _monthname[month], year, hh, mm, ss)

def char2Html(_value = None):
    """Realiza el cambio en un texto, de todos los caracteres especiales en UTF-8 y ASCII a su correspondencia en HTML"""
    char2 = {r'\xe1': "&aacute;", \
                  r'\xc1': "&Aacute;", \
                  r'\xe9': "&eacute;", \
                  r'\xc9': "&Eacute;", \
                  r'\xed': "&iacute;", \
                  r'\xcd': "&Iacute;", \
                  r'\xf3': "&oacute;", \
                  r'\xd3': "&Oacute;", \
                  r'\xfa': "&uacute;", \
                  r'\xda': "&Uacute;", \
                  r'\xf1': "&ntilde;", \
                  r'\xd1': "&Ntilde;", \
                  r'\xfc': "&uuml;", \
                  r'\xdc': "&Uuml;", \
                  r'\x7e': "&tilde;", \
                  r'\x3c': "&lt;", \
                  r'\x3e': "&gt;", \
                  r'\xa3': "&pound;", \
                  r'\x80': "&euro;", \
                  r'\x24': "$", \
                  r'\xa5': "&yen;", \
                  'á': "&aacute;", \
                  'Á': "&Aacute;", \
                  'é': "&eacute;", \
                  'É': "&Eacute;", \
                  'í': "&iacute;", \
                  'Í': "&Iacute;", \
                  'ó': "&oacute;", \
                  'Ó': "&Oacute;", \
                  'ú': "&uacute;", \
                  'Ú': "&Uacute;", \
                  'ü': "&uuml;", \
                  'Ü': "&Uuml;", \
                  'ñ': "&ntilde;", \
                  'Ñ': "&Ntilde;", \
                  'ç': "&ccedil;", \
                  'Ç': "&Ccedil;", \
                  '~': "&tilde;", \
                  '<': "&lt;", \
                  '>': "&gt;", \
                  '£': "&pound;", \
                  '€': "&euro;", \
                  ' EURO ': " &euro; ", \
                  ' euro ': " &euro; ", \
                  ' EUROS ': " &euro; ", \
                  ' euros ': " &euro; ", \
                  '¥': "&yen;", \
                  '&': "&amp;", \
                  '"': "&quot;", \
                  "'": "&apos;" \
                  }
    for n in list(char2.keys()):
        _value = re.sub(n,char2[n],_value)
    return _value

'''
DEBUG - debug message
INFO - info message
WARNING - warn message
ERROR - error message
CRITICAL - critical message
'''
LOG_LEVEL_VALUE = {'DEBUG':10,'INFO':20,'WARNING':30,'ERROR':40,'CRITICAL':50}
if str(os.environ.get('PYMETRICK_LOG_LEVEL',None)).upper() in LOG_LEVEL_VALUE:
    LOG_LEVEL = LOG_LEVEL_VALUE[str(os.environ.get('PYMETRICK_LOG_LEVEL')).upper()]
else:
    LOG_LEVEL = 30
LOG_FILENAME = '-'.join([os.path.abspath(__file__).split(os.sep)[len(os.path.abspath(__file__).split(os.sep))-1],])[:-3]
LOG = logging.getLogger(LOG_FILENAME)

if 'LD_LIBRARY_PATH' in list(os.environ.keys()):
    # CGI environment
    sys.stdout = sys.stderr
    logging.basicConfig(stream = sys.stderr, level=LOG_LEVEL, format='%(filename)s[line:%(lineno)d] - %(levelname)s - %(message)s')
else:
    # not CGI environment
    logging.basicConfig(stream=sys.stderr)
    hdlr = logging.handlers.RotatingFileHandler(filename=LOG_FILENAME+'.log',mode='a', encoding='utf-8', maxBytes=1048576, backupCount=3)
    formatter = logging.Formatter('%(asctime)s - %(filename)s[line:%(lineno)d] - %(levelname)s - %(message)s')
    hdlr.setFormatter(formatter)
    LOG.addHandler(hdlr)
    LOG.setLevel(LOG_LEVEL)

#=== CLASS HTML5 ===============================================================

class Html5(object):
    """Realiza objetos HTML5 con una estructura completa HTML,HEAD,BODY,FOOTER """

    def __init__( self ):
        """Inicializa la clase Html5"""

        self._text = ''
        self.treeHtml = []
        self.treeHead = []
        self.treeBody = []


    def tagHtml(self, content='lang="es"'):

        self._text = content.strip()
        tokenHtml = {"doctype":"<!DOCTYPE html>",
                          "html":"<html [[content]]>"}
        self.treeHtml.append(tokenHtml["doctype"])
        self.treeHtml.append(tokenHtml["html"].replace('[[content]]',self._text))
        self._text = ''

    def tagHead(self):

        tokenHead = {"head":"<head>"}
        self.treeHead.append(tokenHead['head'])


    def addHead( self, *args, **kwargs):
        '''
           tag, content, size, type_content
           Para etiqueta stylesheet
           type_content  all, aural, braille, embossed, handheld, print, projection, screen, tty, tv

           Para etiqueta script
           type text/javascript

           Para etiqueta apple-touch-icon-precomposed
           size
                 144x144 , 114x114, 72x72

           type_content
                 Formato png: image/png
                 Formato gif: image/gif
                 Formato jpeg (extension jpg o jpeg): image/jpeg
                 Formato ico  image/x-icon

           Ejemplos meta :
           <meta name="HandheldFriendly" content="True">
           <meta name="MobileOptimized" content="320">
           <meta name="viewport" content="width=device-width, initial-scale=1.0, maximum-scale=1.0, user-scalable=0">

           <base href="http://www.w3schools.com/images/" target="_blank">
        '''
        """global treeHead"""

        _metaTags = {'tag':'','content':'','size':'','type_content':''}
        _metaList = ['tag','content','size','type_content']
        if args:
            for n in range(len(args)):
                _metaTags[_metaList[n]] = args[n]
        if kwargs:
            for n,m in list(kwargs.items()):
                _metaTags[n]=m

        nodeTag = {"title":'<title>[[content]]</title>',
                   "DC.title":'<meta name="DC.title" content="[[content]]">',
                   "description":'<meta name="description" content="[[content]]">',
                   "keywords":'<meta name="keywords" content="[[content]]">',
                   "author":'<meta name="author" content="[[content]]">',
                   "generator":'<meta name="generator" content="[[content]]">',
                   "application-name":'<meta name="application-name" content="[[content]]">',
                   "content-type":'<meta http-equiv="content-type" content="[[content]]">',
                   "default-style":'<meta http-equiv="default-style" content="[[content]]">',
                   "language":'<meta http-equiv="content-language" content="[[content]]">',
                   "refresh":'<meta http-equiv="refresh" content="[[content]]">',
                   "cleartype":'<meta http-equiv="cleartype" content="[[content]]">',
                   "cache-control":'<meta http-equiv="Cache-control" content="[[content]]">',
                   "handheldfriendly":'<meta name="HandheldFriendly" content="[[content]]">',
                   "mobileoptimized":'<meta name="MobileOptimized" content="[[content]]">',
                   "viewport":'<meta name="viewport" content="[[content]]">',
                   "robots":'<meta name="robots" content="[[content]]">',
                   "ICBM":'<meta name="ICBM" content="[[content]]">',
                   "geo.position":'<meta name="geo.position" content="[[content]]">',
                   "geo.placename":'<meta name="geo.placename" content="[[content]]">',
                   "geo.region":'<meta name="geo.region" content="[[content]]">',
                   "google-site-verification":'<meta name="google-site-verification" content="[[content]]">',
                   "bing-site-verification":'<meta name="msvalidate.01" content="[[content]]">',
                   "yandex-site-verification":'<meta name="yandex-verification" content="[[content]]">',
                   "pinterest-site-verification":'<meta name="p:domain_verify" content="[[content]]">',
                   "authenticity-token":'<meta name="authenticity-token" content="[[content]]">',
                   "charset":'<meta charset="[[content]]">',
                   "format-detection":'<meta name="format-detection" content="[[content]]">',
                   "stylesheet":'<link rel="stylesheet" type="text/css" href="[[content]]" media="[[type]]"/>',
                   "style":'<style type="text/css">[[content]]</style>',
                   "script":'<script src="[[content]]" type="[[type]]" ></script>',
                   "apple-touch-icon-precomposed":'<link rel="apple-touch-icon-precomposed" sizes="[[size]]" href="[[content]]">',
                   "apple-touch-icon":'<link rel="apple-touch-icon" href="[[content]]">',
                   "apple-touch-icon_size":'<link rel="apple-touch-icon" sizes="[[size]]" href="[[content]]">',
                   "shortcut_icon":'<link rel="shortcut icon" href="[[content]]">',
                   "shortcut_icon_type":'<link rel="shortcut icon" href="[[content]]" type="[[type]]">',
                   "apple-touch-startup-image":'<link rel="apple-touch-startup-image" href="[[content]]">',
                   "code":'[[content]]'
                   }

        if _metaTags['tag'] in nodeTag:
            self.treeHead.append(nodeTag[_metaTags['tag']].replace('[[content]]',_metaTags['content']).replace('[[size]]',_metaTags['size']).replace('[[type]]',_metaTags['type_content']))
        else:
            '''No existen ningun nodo que identifique su proceso'''
            self.treeHead.append(_metaTags)


    def gettreeHead(self):
        """Devuelve el dict que contiene los datos de Head"""
        return self.treeHead

    def tagBody(self,content=''):
        """Introduce en el objeto HTML5 la etiqueta <BODY> e inicializa su estructura"""

        self._text = content.strip()
        tokenBody = {}
        tokenBody = {"body":"<body [[content]]>"}
        self.treeBody.append(tokenBody["body"].replace('[[content]]',self._text))
        self._text = ''

    def addBody(self,*args,**kwargs):
        """Incorpora elementos a la estructura de <BODY> del objeto HTML5"""

        tokenBody = {}
        if args:
            for n in args:
                self.treeBody.append(n)
        if kwargs:
            for n,m in list(kwargs.items()):
                '''pendiente de gestionar'''

    def gettreeBody(self):
        """Devuelve el dict que contiene los datos de Body"""

        return self.treeBody

    def getHtml(self):
        """Recupera la información del objeto HTML5 en formato HTML"""

        try:
            self._text = ''
            for x in self.treeHtml:
                self._text += "%s\r" % x
            if len(self.treeHead)>0:
                for x in self.treeHead:
                    self._text += "%s\r" % x.replace('&#61;','=')
                if re.search(r'<head',self._text.replace(' ','')):
                    self._text += '</head>\r'
            if len(self.treeBody)>0:
                for x in self.treeBody:
                    self._text += "%s\r" % x
                if re.search(r'<body',self._text.replace(' ','')):
                    self._text += '</body>\r'
            if re.search(r'<html',self._text.replace(' ','')):
                    self._text += '</html>\r'
        except Exception as e:
            tb = sys.exc_info()[2]
            LOG.error('Error en Html5.getHtml <%s> in line %s !!!' % (repr(e),tb.tb_lineno))
        finally:
            if not self._text:
                self._text = "<!DOCTYPE HTML><html><body><div>NO EXISTE INFORMACION</div></body></html>"
            return self._text

    def gettreeHtml(self):
        """Devuelve el dict que contiene los datos de Html"""

        return self.treeHtml

    def escapeRegexp(string):
        '''Escape special characters'''

        specialCharacters = (".", "^", "$", "*", "+", "?", "{", "}",
                          "[", "]", "|", "(", ")", "-")
        for char in specialCharacters:
            string = string.replace(char, "\\" + char)
        return string

    def hexToInt(hex_str):
        '''Transforma en hexadecimal'''
        return int(hex_str, 16)

    def __del__(self):
        '''Eliminar objeto y objetos dependientes'''
        LOG.debug("Html5.del ha eliminado el objeto")

#=== CLASS HTML TEMPLATE========================================================

class htmlTemplate(object):
    """Tratamiento de templates para generar paginas html"""

    """
    Convenciones para ficheros con extension .tpl
    Los comandos de control que se indican a continuacion deben comenzar in lines separadas.
    Comentarios       <!-- Esto es un comentario --> los comentarios no se insertan en el fichero resultado
    Iniciar un bloque {%block body%}  cerrar un bloque  {%endblock body%}
    Bloques posibles                  html,head,body,footer,article,section,aside,form,header,hgroup,table,nav
    Iniciar y cerrar funciones {%for i in ies%}  {%if i%} {%else%} {%endif%} {%endfor%}
    Variables o funciones  {{fecha_del_dia}} o {{dia_semana()}}  sin espacios
    Insertar informacion de forma exacta   {%literal%}  {%endliteral%}
    Incluir una plantilla externa {%include fichero_template.tpl%}   Se procesa sobrecargando la clase
    Incluir datos de un fichero externo de forma literal {%merge fichero_externo.txt%} los datos se importan
    tal cual, igual que si la informacion fuera insertada entre las etiquetas {%literal%}...{%endliteral%}
    Todo dato que se introduzca entre < y > se comprobara como elemento HTML valido y se insertara en el codigo
    Condiciones posibles
         {% if condicion %}
          ...
         {% else %}
          ...
         {% endif %}


         {% if not condicion %}
         ...
         {% endif %}

     Bucles posibles
          {% for valor in lista_valores %}
          ...
          {% endfor %}

    Como insertar valores en el fichero resultado :
    **kwargs es un parametro opcional, que puede contener un diccionario, con una estructura clave:valor, la clave
    se buscara dentro del codigo como una variable {{variable}} y ser sustituida por el valor.
    """

    #Declaracion de variables universales de clase
    UNJOIN = '|'
    _stack = []
    _tpl = None
    _deep = 0
    _path_file = ''

    def __init__(self,template_file, path_file=''):
        # Es posible que la clase deba interpretar diversas plantillas de forma recursiva a partir de la plantilla principal,
        # la profundidad de plantillas llamadas desde la plantilla principal se mide con _deep
        # Si _deep=1 el proceso estará interpretando la plantilla principal.
        htmlTemplate._deep += 1

        
        if len(path_file)>0:
            htmlTemplate._path_file = path_file
            _file = htmlTemplate._path_file + self.__system__(template_file).lower()
        else:
            _file = self.__system__(template_file).lower()
            
        # comprobacion de profundidad recursiva
        LOG.debug('__init__ _deep <%s> !!!' % (str(htmlTemplate._deep)))
        #
        LOG.debug('__init__ _path_file <%s> !!!' % (htmlTemplate._path_file))
        self._tpl_form = ''
        self._tpl_html = ''
        self.parseHtml(_file)
        

    def parseHtml(self,template_file):
        # Interpreta el fichero .tpl

        _file = ''
        linea = ''
        f1 = ''
        _elements = list()
        try:

            _file = self.__system__(template_file).lower()
            LOG.debug('''htmlTemplate.parseHtml template_file %s''' % _file)
            if _file and _file[-3:]=='tpl' and os.path.isfile( _file ):
                with open( _file, mode='r', encoding=sys.getfilesystemencoding(), errors='ignore') as f1:
                # Inicializar objeto html5

                    while True:
                        #Procesar plantilla
                        #Eliminar lineas de comentario <!-- ... -->
                        
                        _ll = f1.readline()
                        while re.search(r'<!--.*?-->\n',_ll):
                            _ll = re.sub(r'<!--.*?-->\n','',_ll)
                            if not _ll or len(_ll)==0:
                                _ll = f1.readline()
                        linea += ' '+_ll
                        #linea += ' '+f1.readline()
                        #Pueden aparecer en la línea Byte Order Mark o marca en little-endian \xEF\xBB\xBF que debemos eliminar
                        linea = linea.replace('\xef\xbb\xbf','')
                        linea = linea.strip()
                        if not linea:
                            #No existen datos y finaliza el proceso
                            break


                        if not re.search(r'^{%literal%}',linea.replace(' ','')):
                            #Buscar etiqueta html
                            if re.search(r'^{%blockhtml',linea.replace(' ','')):
                                htmlTemplate._tpl = Html5()
                                if linea.find(self.UNJOIN)>-1:
                                    linea = linea[linea.find(self.UNJOIN)+1:].replace('{%','').replace('%}','')
                                    htmlTemplate._tpl.tagHtml(linea)
                                else:
                                    htmlTemplate._tpl.tagHtml()
                                linea = ''
                                htmlTemplate._stack.append('html')

                                #Buscar comentarios  <!-- ... -->
                            elif re.search('^<!--',linea) and re.search('-->$',linea.replace(' ','')):
                                #Se eliminan los comentarios para producir un codigo más compacto
                                linea = ''
                                linea += ' '+f1.readline()
                                linea = linea.strip()
                                if linea.__len__()==0:
                                    break

                                """Buscar bloques {%block ....%} y cierre de bloques  {%endblock ...%}
                                los bloques posibles son html, head, body, footer, article, section,
                                aside, form, header, hgroup, table, nav """
                            elif re.search(r'^{%block',linea.replace(' ','')):
                                #Extrae la informacion de bloque
                                self.__block__(linea)
                                linea = ''

                            elif re.search(r'^{%endblock',linea.replace(' ','')):
                                #Extrae la informacion de fin de bloque
                                self.__block__(linea)
                                linea = ''

                            elif re.search(r'^{%if',linea.replace(' ','')) or re.search(r'^{%else',linea.replace(' ','')) or re.search('^{%for',linea.replace(' ','')):
                                #Comprobar comandos if, else y for
                                self.__loop__(linea)
                                linea = ''

                            elif re.search(r'^{%endif',linea.replace(' ','')) or re.search(r'^{%endfor',linea.replace(' ','')):
                                #Comprobar comandos endif y endfor
                                self.__loop__(linea)
                                linea = ''

                                #Buscar importacion de plantillas externas {%include fichero_template.tpl%}
                            elif re.search(r'^{%include',linea.replace(' ','')) and re.search(r'%}$',linea.replace(' ','')):
                                linea = self.__include__(linea)

                                #Buscar importar fichero externo {%merge fichero_externo.txt%} literalmente
                            elif re.search(r'^{%merge',linea.replace(' ','')) and re.search(r'%}$',linea.replace(' ','')):
                                linea = self.__merge__(linea)

                                #Buscar comandos <% .... %>
                            elif re.search(r'^<%',linea.replace(' ','')) and re.search(r'%>$',linea.replace(' ','')):
                                self.__command__(linea)
                                linea = ''

                                #Buscar etiquetas html < >...</ >
                                """ La primera línea que solo contenga etiquetas html debe comenzar con < y la ultima linea debe finalizar con </
                                Las restantes lineas contenidas pueden contener etiquetas html o no.
                                Las etiquetas html con autocierre no se consideran para el proceso """
                            elif re.search(r'^<\w+',linea.replace(' ','')) or re.search(r'^</\w+',linea.replace(' ','')) or re.search(r'</\w+>$',linea.replace(' ','')) or len(_elements)>0:
                                l = re.findall(r'(<\w+|<\w+\s|</\w+)',linea)
                                for i in l:
                                    i= re.sub(r'^<', r'', i)
                                    if i in acceptable_body_elements and i not in self_closing_tag:
                                        #Si es una etiqueta html de apertura, se guardara hasta comprobar la etiqueta html de cierre
                                        _elements.append(i)
                                    else:
                                        i= re.sub(r'^/',r'',i)
                                        if i in acceptable_body_elements and i not in self_closing_tag and len(_elements)>0:
                                            if i == _elements[len(_elements)-1]:
                                                #Se comprueba etiqueta html de cierre y se elimina de la pila, la etiqueta html de apertura
                                                _elements.pop()
                                        else:
                                            if i in self_closing_tag:
                                                #Es una etiqueta html de autocierre y no se considera para su tratamiento
                                                pass
                                linea = '{%literal%}'+linea+'{%endliteral%}'

                        #Buscar informacion literal {%literal%} ... {%endliteral%}
                        if (re.search(r'^{%literal%}',linea.replace(' ','')) and re.search(r'{%endliteral%}$',linea.replace(' ',''))):
                            self.__literal__(linea)
                            linea = ''
                f1.close()
            else:
                #No existe fichero de plantilla
                htmlTemplate._tpl = Html5()
                htmlTemplate._tpl.tagHtml()
                self.__block__('{%block html|lang="es"%}')
                self.__block__('{%block head%}')
                self.__block__('{%block body%}')
                if _file:
                    linea = 'NO EXISTE %s' % _file
                else:
                    linea = 'NO EXISTEN DATOS'
                self.__literal__('{%literal%}<div>'+linea+'</div>{%endliteral%}')
                self.__block__('{%block footer%}')
                self.__block__('{%endblock footer%}')
                self.__block__('{%endblock body%}')
                self.__block__(linea)

        except Exception as e:
            tb = sys.exc_info()[2]
            LOG.error('Error en htmlTemplate.parseHtml <%s> in line %s !!!' % (repr(e),tb.tb_lineno))
        finally:
            if _file and os.path.isfile(_file):
                f1.close()


    def get(self):

        _vars_ = []
        _tpl_html_copy_ = ''
        try:
            if not self._tpl_html:
                self._tpl_html = htmlTemplate._tpl.getHtml()
                _tpl_html_copy_ = self._tpl_html
            else:
                _tpl_html_copy_ = self._tpl_html
        except Exception as e:
            tb = sys.exc_info()[2]
            LOG.error('Template -> Error en htmlTemplate.get <%s> in line %s !!!' % (repr(e),tb.tb_lineno))
        finally:
            #Solo finaliza de procesar el codigo html si _deep<=1 o procesando la plantilla principal
            if htmlTemplate._deep<=1:
                htmlTemplate._stack = []
                htmlTemplate._tpl = ''
                htmlTemplate._deep = 0
                # _tpl_html_copy_ += '</html>'
            return _tpl_html_copy_

    def __block__(self, linea):

        _text = ''
        _tag = ''
        try:
            if re.search('^{%block',linea.replace(' ','')):
                if re.search('^{%blockhead%}',linea.replace(' ','')):
                    htmlTemplate._tpl.tagHead()
                    htmlTemplate._stack.append('head')
                elif re.search('^{%blockbody',linea.replace(' ','')):
                    if linea.find(self.UNJOIN)>-1:
                        htmlTemplate._tpl.tagBody(linea[linea.find(self.UNJOIN)+1:].replace('{%','').replace('%}',''))
                    else:
                        htmlTemplate._tpl.tagBody()
                    htmlTemplate._stack.append('body')
                else:
                    _tag = linea[7:linea.find('%}')].strip()
                    if linea.find(self.UNJOIN)>-1:
                        _tag = _tag[:_tag.find(self.UNJOIN)].replace(' ','')
                        _text = linea[linea.find(self.UNJOIN)+1:].replace('{%','').replace('%}','')
                        # incorpora a etiquetas block distintas de html, head, body y form, los atributos recibidos
                        _tag = ' '.join((_tag,_text))
                    else:
                        _text = ''
                    _tag = _tag.lower()
                    _text_list = []
                    _text_dict = {}
                    if len(htmlTemplate._stack)>0:
                        if htmlTemplate._stack[-1]=='body':
                            htmlTemplate._stack.append(_tag)
                            if htmlTemplate._stack[-1]=='form':
                                self._tpl_form = Form(_text)
                            else:
                                #Apertura de etiquetas block diferentes a html,head,body y form
                                _text_list.append('<%s>' % _tag)
                                htmlTemplate._tpl.addBody(*_text_list,**_text_dict)
            if re.search('^{%endblock',linea.replace(' ','')):
                if re.search('{%endblockhead%}$',linea.replace(' ','')):
                    if len(htmlTemplate._stack)>0:
                        if htmlTemplate._stack[-1] == 'head':
                            htmlTemplate._stack.pop()
                elif re.search('{%endblockbody%}$',linea.replace(' ','')):
                    if len(htmlTemplate._stack)>0:
                        if htmlTemplate._stack[-1] == 'body':
                            htmlTemplate._stack.pop()
                else:
                    if len(htmlTemplate._stack)>0:
                        _text_list = []
                        _text_dict = {}
                        if htmlTemplate._stack[-1] == linea.replace(' ','')[10:linea.replace(' ','').find('%}')]:
                            _tag = linea.replace(' ','')[10:linea.replace(' ','').find('%}')].lower()
                            if htmlTemplate._stack[-1]=='form' and _tag=='form':
                                #Crear lista de valores para incluir en Form
                                _text_list = []
                                _text_list.append(self._tpl_form.getForm())
                                htmlTemplate._tpl.addBody(*_text_list,**_text_dict)
                            else:
                                _text_list.append('</%s>' % _tag)
                                htmlTemplate._tpl.addBody(*_text_list,**_text_dict)
                            htmlTemplate._stack.pop()
        except Exception as e:
            tb = sys.exc_info()[2]
            LOG.error('Error en htmlTemplate.__block__ <%s> in line %s !!!' % (repr(e),tb.tb_lineno))
        finally:
            pass

    def __loop__(self, linea):

        _tag = None
        _rslt = None
        try:
            if re.search('^{%if',linea.replace(' ','')):
                _text = u(linea.replace('{%','').replace('%}',':\n\r').strip())
                _text_tab += 4
            elif re.search('^{%for',linea.replace(' ','')):
                _text = u(linea.replace('{%','').replace('%}',':\n\r').strip())
                _text_tab += 4
            elif re.search('^{%else',linea.replace(' ','')):
                _text +='else:\n'
            elif re.search('^{%endif',linea.replace(' ','')):
                _text_tab -= 4
                if len(_text)>0:
                    _tag = compile(_text,'','exec')
                    exec(_tag)
                    htmlTemplate._tpl = _rslt
                    _text = None
                    _text_tab = 0
            elif re.search('^{%endfor',linea.replace(' ','')):
                _text_tab -= 4
                if len(_text)>0:
                    _tag = compile(_text,'','exec')
                    exec(_tag)
                    htmlTemplate._tpl = _rslt
                    _text = None
                    _text_tab = 0
            else:
                _text += '_rslt += "'+' '*_text_tab+linea.strip()+'\n"'

        except Exception as e:
            tb = sys.exc_info()[2]
            LOG.error('Error en htmlTemplate.__loop__ <%s> in line %s !!!' % (repr(e),tb.tb_lineno))
        finally:
            pass


    def __merge__(self, linea):

        _text = '{%literal%}'
        _file = ''
        try:
            if re.search('^{%merge',linea.replace(' ','')):
                #Carga fichero sin tratar contenido, como literal
                _file = self.__system__(linea.replace(' ','')[7:linea.find('%}')-1].strip())
                LOG.debug('htmlTemplate.__merge__ _file <%s> !!!' % (htmlTemplate._path_file + _file,))
                
                if os.path.isfile(htmlTemplate._path_file + _file):
                    with open(htmlTemplate._path_file + _file, mode="r", encoding=sys.getfilesystemencoding(), errors='ignore') as f2:
                        _linea2 = f2.readline()
                        _text += _linea2
                        while True:
                            _linea2 = f2.readline()
                            _text += _linea2
                            if _linea2 == "":
                                break
                        f2.close()
                        _linea2 = ''
                    f2.close()
                else:
                    _text += '<div>NO EXISTE %s</div>' % _file

        except Exception as e:
            tb = sys.exc_info()[2]
            LOG.error('Error en htmlTemplate.__merge__ <%s> in line %s !!!' % (repr(e),tb.tb_lineno))
        finally:
            _text += '{%endliteral%}'
            return _text


    def __include__(self, linea):

        _tpl_include = ''
        _file = ''
        _text = ''
        try:
            if re.search('^{%include',linea.replace(' ','')):
                _file = self.__system__(linea.replace(' ','')[9:linea.find('%}')-1].strip())
                LOG.debug('htmlTemplate.__include__ _file <%s> !!!' % (htmlTemplate._path_file + _file,))

                if _file and _file[-3:].lower()=='tpl':
                    if os.path.isfile(htmlTemplate._path_file  + _file):
                        _tpl_include = htmlTemplate(_file , htmlTemplate._path_file )
                        '''_text = _tpl_include.get()'''
                        htmlTemplate._deep -= 1
                    else:
                        #No encuentra el fichero de carga
                        _text += '<div>NO EXISTE %s</div>' % _file
        except Exception as e:
            tb = sys.exc_info()[2]
            LOG.error('Error en htmlTemplate.__include__ <%s> in line %s !!!' % (repr(e),tb.tb_lineno))
        finally:
            if len(_text)>0:
                return ''.join(['{%literal%}', _text,'{%endliteral%}'])
            else:
                return ''

    def __literal__(self,linea):

        _text = ''
        try:
            _text += linea
            _text = _text.replace('{%literal%}','').replace('{%endliteral%}','')
            _text_dict = {}
            _text_list = []
            if len(htmlTemplate._stack)>0:
                if htmlTemplate._stack[-1] == 'head':
                    #Crear lista de valores, precedidos por 'code' para Head
                    _text_list.append('code')
                    _text_list.append(_text)
                    htmlTemplate._tpl.addHead(*_text_list,**_text_dict)
                elif htmlTemplate._stack[-1] == 'form':
                    #Crear lista de valores para Form
                    htmlTemplate._tpl.addForm(*_text_list,**_text_dict)
                else:
                    #Crear lista de valores para Body, por defecto
                    _text_list.append(_text)
                    htmlTemplate._tpl.addBody(*_text_list,**_text_dict)
            else:
                #Por defecto, crea lista de valores para Body
                _text_list.append(_text)
                #Si no es una plantilla completa html, se inicializa el objeto HTML5
                if type(htmlTemplate._tpl).__name__ != "Html5":
                    htmlTemplate._tpl = Html5()
                htmlTemplate._tpl.addBody(*_text_list,**_text_dict)
                _text = ''
        except Exception as e:
            tb = sys.exc_info()[2]
            LOG.error('Error en htmlTemplate.__literal__ <%s> in line %s !!!' % (repr(e),tb.tb_lineno))
        finally:
            pass


    def __command__(self, linea):

        _text = ''
        try:
            #Cargar contenido tratado con parametros
            _text = linea
            _text_split = _text.replace('<%','').replace('%>','').split(self.UNJOIN)
            _text_dict = {}
            _text_list = []
            for n in _text_split:
                n = n.strip()
                pos_x = n.find('=')
                if pos_x > -1:
                    #Crear un diccionario con valores con clave
                    _text_dict[n[0:pos_x]]=n[pos_x+1:].replace('"','')
                else:
                    #Crear una lista con valores sin clave
                    _text_list.append(n)
            if len(htmlTemplate._stack)>0:
                if htmlTemplate._stack[-1] == 'head':
                    htmlTemplate._tpl.addHead(*_text_list,**_text_dict)
                elif htmlTemplate._stack[-1] == 'form':
                    self._tpl_form.addForm(*_text_list,**_text_dict)
                else:
                    htmlTemplate._tpl.addBody(*_text_list,**_text_dict)
            else:
                #Por defecto, crea lista de valores para Body
                _text_list.append(_text)
                htmlTemplate._tpl.addBody(*_text_list,**_text_dict)
                _text = ''
        except Exception as e:
            tb = sys.exc_info()[2]
            LOG.error('Error en htmlTemplate.__command__ <%s> in line %s !!!' % (repr(e),tb.tb_lineno))

    def setData(self,data=None):
        '''A partir de un diccionario, sustituira las claves por valores
        las claves a sustituir se encuentran entre {{ y }}. La modificacion
        se produce en una copia de la plantilla, por tanto, la plantilla no
        se ve afectada por los cambios.
        Las claves no sustituidas se modificaran a un espacio
        '''

        try:
            __strLeft = "{{"
            __strRight = "}}"
            __strClear = ""
            tplData = self.get()
            if type(data).__name__!='dict':
                data = dict()
            for n in list(data.keys()):
                data[n] = data[n]
                LOG.debug('data %s %s' % (data[n],type(data[n])))
                tplData = tplData.replace('%s%s%s' % (__strLeft,n,__strRight),"%s" % data[n])
            tplData = re.sub("\{{2}[\w]+\}{2}",__strClear,tplData)
            # compresor de todo el contenido HTML
            '''
            tplData = re.sub('<!--(.*?)-->','',tplData)                 # eliminar comentarios html <!--
            tplData = re.sub('\/\*(.*?)\*\/','',tplData)                # eliminar comentarios javascript multilinea /*
            '''
            if HTML_COMPRESS:
                tplData = re.sub('>\s<','><',re.sub('\s\s+','\s',tplData))   # varios espacios o espacios entre > <
                tplData = re.sub('"\s+/>','"/>',tplData)                     # espacios entre " />
                no_space = [['\n',''],['\r',''],['=\s','='],['\s=','='],[':\s',':'],['\s:',':'],[';\s',';'],['\s;',';'],['/\*(.*?)\*/',''],['"\s>','">'],["'\s<","'<"],['\s/>','/>']]
                # ['/\*(.*?)\*/',''],['\)[\s]{','){']
                for n in no_space:
                    tplData = re.sub(n[0],n[1],tplData)
                # reemplazar \s{ {\s \s} }\s  \s(  (\s  \s) )\s
                no_space = [[' {','{'],['{ ','{'],[' }','}'],['} ','}'],[' (','('],['( ','('],[' )',')'],[') ',')']]
                for n in no_space:
                    tplData = tplData.replace(n[0],n[1])
            # finaliza compresor de todo el codigo HTML
            if os.environ.get('PYMETRICK_LOG',None) == 'logging.DEBUG':
                with open('htmlTemp.txt',mode='w', encoding="utf-8") as file_t:
                    file_t.write(tplData)
                LOG.debug('TIPO DE DATOS %s' % (self.errorTags(tplData),))
            return tplData
        except Exception as e:
            tb = sys.exc_info()[2]
            LOG.error('Error en htmlTemplate.setData <%s> in line %s !!!' % (repr(e),tb.tb_lineno))

    def errorTags(self,text):
        '''Comprobar etiquetas correctas'''
        try:
            d_none = dict()
            i = re.findall(r"<[^\/][a-zA-Z0-9]*?[\s,>]",text,re.M)
            d_init = dict()
            for n in i:
                if n.strip() not in list(d_init.keys()):
                    d_init[n.strip()] = 1
                else:
                    d_init[n.strip()] += 1
            e = re.findall(r"<\/[a-zA-Z0-9]*?>",text,re.M)
            d_end = dict()
            for n in e:
                if n.strip() not in list(d_end.keys()):
                    d_end[n.strip()] = 1
                else:
                    d_end[n.strip()] += 1
            # eliminar claves apertura con >
            for n in list(d_init.keys()):
                if len(n) != len(n.replace('>','')):
                    value = d_init[n]
                    if n.replace('>','') in list(d_init.keys()):
                        d_init[n.replace('>','')] = d_init[n.replace('>','')] + value
                    else:
                        d_init[n.replace('>','')] = value
                    del d_init[n]
            # comparar apertura y terminacion
            for n in list(d_end.keys()):
                m = n.replace('/','').replace('>','')
                if m in list(d_init.keys()):
                    # acumula las diferencias
                    if d_init[m] != d_end[n]:
                        d_none[m] = [d_init[m],d_end[n]]
                        # comprueba errores
                        for match in re.finditer(r""+m+"\s", text):
                            s = match.start()
                            e = match.end()
                        for match in re.finditer(r""+n+"\s", text):
                            s = match.start()
                            e = match.end()
                    # eliminar key apertura
                    del d_init[m]
                else:
                    # terminacion no tiene inicio
                    d_none[m] = [0,d_end[n]]
            if len(d_init)>0:
                for m in list(d_init.keys()):
                    # apertura sin terminacion
                    d_none[m] = [d_init[m],0]
            return d_none
        except Exception as e:
            tb = sys.exc_info()[2]
            LOG.error('Error en htmlTemplate.setData <%s> in line %s !!!' % (repr(e),tb.tb_lineno))
            return {}


    def __system__(self,linea):

        __path__ = ''
        __os__ = ''
        try:
            if linea:

                __os__ = os.name
                if __os__ == 'posix':
                    __path__ = linea.replace("\\",os.sep)
                elif __os__ == 'nt':
                    __path__ = linea.replace("\\",os.sep)
                else:
                    #Sistema operativo desconocido
                    __path__ = linea.strip()
        except Exception as e:
            tb = sys.exc_info()[2]
            LOG.error('Error en htmlTemplate.__system__ <%s> in line %s !!!' % (repr(e),tb.tb_lineno))
        finally:
            return __path__

    def __del__(self):
        '''Eliminar objeto y objetos dependientes'''
        LOG.debug("Template.del ha eliminado el objeto")

if __name__ == "__main__":
    print ('''copyright {0}'''.format( __copyright__))
    print ('''license {0}'''.format( __license__))
    print ('''version {0}'''.format( __version__))
    if len(sys.argv) < 2:
        sys.stderr.write("for help use -h o --help")
    elif sys.argv[1]=='-h' or sys.argv[1]=='--help':
        print ('''
        Puede cargar una plantilla que siempre debe tener una extensión .tpl :
        página = htmlTemplate('e:/mipymetrick/index.tpl')
        posteriormente recuperará los datos html creados con :
        pagina.get()
        ''')




def calcula_dc(secuencia):
    """Calcula el dígito de control de una CCC.
    Recibe una lista con 10 numeros enteros y devuelve el DC
    correspondiente"""

    pesos = [1, 2, 4, 8, 5, 10, 9, 7, 3, 6]
    aux = []
    for i in range(10):
        aux.append(secuencia[i]*pesos[i])
    resto = 11 - sum(aux)%11
    if resto == 10:
        return 1
    elif resto == 11:
        return 0
    else:
        return resto


import random
from core import _random_words

sociedades = ('SL', 'SA', 'SLU', 'CB', '')
_random_sociedades = lambda:random.choice(sociedades)

def genera_NIF():
    """
    El nif corresponde a un número de identificación para
    individuos, se compone de un número (el DNI) seguido
    por una letra
    """
    num = random.randint(1000000, 100000000)
    nif_control = 'TRWAGMYFPDXBNJZSQVHLCKE'
    checksum = nif_control[int(num)%23]
    return "%s%s" % (num, checksum)

def genera_CIF():
    """
    El CIF se corresponde con el identificador para las
    empresas, es de la forma letra+numero

    http://es.wikipedia.org/wiki/C%C3%B3digo_de_identificaci%C3%B3n_fiscal

    A la hora de generar los CIF priorizaremos las sociedadeS
    anónimas y las limitadas.

    """

    def cif_get_checksum(number, tipo):
        cif_control = 'JABCDEFGHI'
        s1 = sum([int(digit) for pos, digit in enumerate(number) if int(pos) % 2])
        s2 = sum([sum([int(unit) for unit in str(int(digit) * 2)]) for pos, digit in enumerate(number) if not int(pos) % 2])
        num =  (10 - ((s1 + s2) % 10)) % 10
        if tipo in ['A', 'B', 'E', 'H']:
            return num
        else:
            return cif_control[num]
    cifs_habituales = 'AB'
    cif_types = 'CDEFGHKLMNPQS'
    tipo = random.randint(0, 100)
    # cambiamos de distribución uniforme a distribución por peso
    letra = random.choice(cifs_habituales) if tipo < 80 else \
        random.choice(cif_types)
    numero = "%02d%05d" % (random.randint(1, 100),
        random.randint(1000, 100000))
    chk = cif_get_checksum(numero, letra)
    return "%(letra)s%(numero)s%(control)s" % \
        {'letra': letra,
         'numero': numero,
         'control': chk
         }

def nombre_empresa():
    """
    Devuelve una tupla compuesta por el nombre
    de la empresa y su tipo
    """
    num = random.randrange(2, 4)
    empresa = _random_words(num).upper()
    empesa_full = empresa
    x = _random_sociedades()
    return empresa, x



if __name__=='__main__':
    pass
    '''
    print "CIFs"
    print "===="
    for i in range(0, 10):
        print genera_CIF()
    print

    print "NIFs"
    print "===="
    for i in range(0, 10):
        print genera_NIF()

    print
    print "EMPRESAS"
    print "========"
    for i in range(0, 10):
        print " ".join(nombre_empresa())

    # Para cuenta bancaria

    ccc = raw_input("Introduce el CCC: ").split('-')
    entidadyoficina = [0, 0]
    for i in range(2):
        for j in range(4):
            entidadyoficina.append(int(ccc[i][j]))
    cuenta = []
    for i in range(10):
        cuenta.append(int(ccc[3][i]))
    dc = str(calcula_dc(entidadyoficina))+str(calcula_dc(cuenta))
    if dc == ccc[2]:
        print 'CCC verificado'
    else:
        print 'Hay un error en el CCC'
    '''


#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import random
import Image
import ImageFont
import ImageDraw
import ImageFilter
import gtk
import threading
from subprocess import Popen, PIPE, STDOUT
 
gtk.gdk.threads_init()
#--------------------------imagen construir--------------------------------
def gen_random_word(wordLen=6):
    allowedChars = "abcdefghijklmnopqrstuvwzyzABCDEFGHIJKLMNOPQRSTUVWZYZ0123456789"
    word = ""
    for i in range(0, wordLen):
        word = word + allowedChars[random.randint(0,0xffffff) % len(allowedChars)]
    return word
    
def gen_captcha(text, fnt, fnt_sz, file_name, fmt='JPEG'):      
    fgcolor = random.randint(0,1)
    bgcolor = fgcolor ^ 0xffffff
    font = ImageFont.truetype(fnt,fnt_sz)
    dim = font.getsize(text)
    im = Image.new('RGB', (dim[0]+5,dim[1]+5), bgcolor)
    d = ImageDraw.Draw(im)
    x, y = im.size
    r = random.randint
    for num in range(100):
        d.rectangle((r(0,x),r(0,y),r(0,x),r(0,y)),fill=r(0,0xffffff))
    d.text((3,3), text, font=font, fill=fgcolor)
    im = im.filter(ImageFilter.EDGE_ENHANCE_MORE)
    im.save(file_name, format=fmt)
#--------------------------------------------------------------------------
class PackBox1:
    
    def __init__(self):
        window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        window.set_title("-* C A N A I M A *-")
        window.set_border_width(10)
        window.set_size_request(550, 200)
        window.set_resizable(False)
 
        
        def imagen_box2(homogeneous, spacing, expand, fill, padding):
    
            caja = gtk.HBox(homogeneous, spacing)
            caja.set_border_width(5)
    
            image = gtk.Image()
            image.set_from_file('test.jpg')
            caja.pack_start(image, gtk.TRUE, gtk.FALSE,0)
            image.show()
    
            return caja
        
        def make_box1(homogeneous, spacing, expand, fill, padding):
            
            def clic_boton(self):
                hilo = threading.Thread(target=validate, args=(self))
                hilo.start()
            
            def validate(self, data=None):
                                        
            
                if  texto.get_text() != word:
                
                    md=gtk.MessageDialog(parent=None, flags=0, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_CLOSE, message_format="El valor introducido no coincide con el captcha intente de nuevo")
                    md.run()
                    md.destroy()
                    
 
                else:   
                    md=gtk.MessageDialog(parent=None, flags=0, buttons=gtk.BUTTONS_OK, message_format="El valor introducido es Correcto")
                    md.run()
                    md.destroy()        
    
            caja = gtk.HBox(homogeneous, spacing)
            caja.set_border_width(10)
            
            etiqueta = gtk.Label("Introduzca el valor")
            #etiqueta.set_alignment(0,0)
            caja.pack_start(etiqueta, False, False, 30)
            etiqueta.show()
   
            texto = gtk.Entry(10)
            #texto.connect("activate", enter_callback)
            caja.pack_start(texto, False, False, 10)
            texto.show()
   
            boton = gtk.Button(stock=gtk.STOCK_OK)
            boton.connect("clicked", validate)
            caja.pack_start(boton, True, True, 40)
            boton.show()
            return caja
        
        def make_box3(homogeneous, spacing, expand, fill, padding):
   
            caja = gtk.HBox(homogeneous, spacing)
            caja.set_border_width(10)
            
            boton = gtk.Button(stock=gtk.STOCK_CLOSE)
            boton.connect("clicked", gtk.mainquit)
            caja.pack_start(boton, gtk.TRUE, gtk.TRUE, 70)
            boton.show()
            return caja
        
        box1 = gtk.VBox(False, 0)
        
        box2 = imagen_box2(False, 0, False, False,0)
        box1.pack_start(box2, False, False, 0)
        box2.show()
        
        box2 = make_box1(False, 0, False, False,0)
        box1.pack_start(box2, False, False, 0)
        box2.show()
        
        box2 = make_box3(False, 0, False, False,0)
        box1.pack_start(box2, False, False, 20)
        box2.show()
        
        box1.show()
        window.add(box1)
        window.show()
    
def main():
    gtk.main()
    return 0
            
if __name__ == '__main__':
    word = gen_random_word()
    gen_captcha(word.strip(), '/usr/share/fonts/truetype/freefont/FreeSansBoldOblique.ttf', 25, "test.jpg")
    packbox1 = PackBox1()
    main()


#!/usr/bin/python
# -*- coding: utf-8 *-*

# This file is part of Pymetrick.  The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.

"""Modulo para generar sitemap"""

try:
    from pymetrick.version import __author__, __version__, __license__, __copyright__
except ImportError:
    raise ImportError(
        'Failed to find (autogenerated) version.py '
    )

__date__ = '2019-09-21'
__credits__ = ''
__text__ = 'Generador de codigo en fichero sitemap'
__file__ = 'sitemap.py'

import sys
# sin bytecode
sys.dont_write_bytecode = True
import os
import re
import logging
import logging.handlers

# For Python 3.6 and later
import urllib.request


# Available Sitemap types
SITEMAP_TYPES = ['web', 'mobile', 'news']
# General Sitemap tags
GENERAL_SITEMAP_TAGS = ['loc', 'changefreq', 'priority', 'lastmod']

# News specific tags
NEWS_SPECIFIC_TAGS = ['keywords', 'publication_date', 'stock_tickers']

# News Sitemap tags
NEWS_SITEMAP_TAGS = GENERAL_SITEMAP_TAGS + NEWS_SPECIFIC_TAGS

# Maximum number of urls in each sitemap, before next Sitemap is created
MAXURLS_PER_SITEMAP = 50000

# Suffix on a Sitemap index file
SITEINDEX_SUFFIX = '_sitemap.xml'

# Regular expressions tried for extracting URLs from access logs.
ACCESSLOG_CLF_PATTERN = re.compile(r'.+\s+"([^\s]+)\s+([^\s]+)\s+HTTP/\d+\.\d+"\s+200\s+.*')

# XML formats
GENERAL_SITEINDEX_HEADER = \
  '<?xml version="1.0" encoding="UTF-8"?>\n' \
  '<sitemapindex\n' \
  ' xmlns="http://www.sitemaps.org/schemas/sitemap/0.9"\n' \
  ' xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"\n' \
  ' xsi:schemaLocation="http://www.sitemaps.org/schemas/sitemap/0.9\n' \
  ' http://www.sitemaps.org/schemas/sitemap/0.9/' \
  'siteindex.xsd">\n'

NEWS_SITEINDEX_HEADER = \
  '<?xml version="1.0" encoding="UTF-8"?>\n' \
  '<sitemapindex\n' \
  ' xmlns="http://www.sitemaps.org/schemas/sitemap/0.9"\n' \
  ' xmlns:news="http://www.google.com/schemas/sitemap-news/0.9"\n' \
  ' xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"\n' \
  ' xsi:schemaLocation="http://www.sitemaps.org/schemas/sitemap/0.9\n' \
  ' http://www.sitemaps.org/schemas/sitemap/0.9/' \
  'siteindex.xsd">\n'

SITEINDEX_FOOTER = '</sitemapindex>\n'
SITEINDEX_ENTRY = \
  ' <sitemap>\n' \
  ' <loc>%(loc)s</loc>\n' \
  ' <lastmod>%(lastmod)s</lastmod>\n' \
  ' </sitemap>\n'
GENERAL_SITEMAP_HEADER = \
  '<?xml version="1.0" encoding="UTF-8"?>\n' \
  '<urlset\n' \
  ' xmlns="http://www.sitemaps.org/schemas/sitemap/0.9"\n' \
  ' xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"\n' \
  ' xsi:schemaLocation="http://www.sitemaps.org/schemas/sitemap/0.9\n' \
  ' http://www.sitemaps.org/schemas/sitemap/0.9/' \
  'sitemap.xsd">\n'

NEWS_SITEMAP_HEADER	= \
  '<?xml version="1.0" encoding="UTF-8"?>\n' \
  '<urlset\n' \
  ' xmlns="http://www.sitemaps.org/schemas/sitemap/0.9"\n' \
  ' xmlns:news="http://www.google.com/schemas/sitemap-news/0.9"\n' \
  ' xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"\n' \
  ' xsi:schemaLocation="http://www.sitemaps.org/schemas/sitemap/0.9\n' \
  ' http://www.sitemaps.org/schemas/sitemap/0.9/' \
  'sitemap.xsd">\n'

MOBILE_SITEMAP_HEADER = \
  '<?xml version="1.0" encoding="UTF-8" ?>\n' \
  '<urlset\n' \
  ' xmlns="http://www.sitemaps.org/schemas/sitemap/0.9"\n' \
  ' xmlns:mobile="http://www.google.com/schemas/sitemap-mobile/1.0">\n' \
  ' <url>\n' \
  '     <loc>%(loc)s</loc>\n' \
  '     <mobile:mobile/>\n' \
  ' </url>\n' \
  '</urlset>\n'

'''
DEBUG - debug message
INFO - info message
WARNING - warn message
ERROR - error message
CRITICAL - critical message
'''
LOG_LEVEL_VALUE = {'DEBUG':10,'INFO':20,'WARNING':30,'ERROR':40,'CRITICAL':50}
if str(os.environ.get('PYMETRICK_LOG_LEVEL',None)).upper() in LOG_LEVEL_VALUE:
    LOG_LEVEL = LOG_LEVEL_VALUE[str(os.environ.get('PYMETRICK_LOG_LEVEL')).upper()]
else:
    LOG_LEVEL = 30
LOG_FILENAME = '-'.join([os.path.abspath(__file__).split(os.sep)[len(os.path.abspath(__file__).split(os.sep))-1],])[:-3]
LOG = logging.getLogger(LOG_FILENAME)

if 'LD_LIBRARY_PATH' in list(os.environ.keys()):
    # CGI environment
    sys.stdout = sys.stderr
    logging.basicConfig(stream = sys.stderr, level=LOG_LEVEL, format='%(filename)s[line:%(lineno)d] - %(levelname)s - %(message)s')
else:
    # not CGI environment
    logging.basicConfig(stream=sys.stderr)
    hdlr = logging.handlers.RotatingFileHandler(filename=LOG_FILENAME+'.log',mode='a', encoding='utf-8', maxBytes=1048576, backupCount=3)
    formatter = logging.Formatter('%(asctime)s - %(filename)s[line:%(lineno)d] - %(levelname)s - %(message)s')
    hdlr.setFormatter(formatter)
    LOG.addHandler(hdlr)
    LOG.setLevel(LOG_LEVEL)

def buscar(url=None, proxyUrl=None, proxyAuth=None, proxyPasswd=None):
    try:

        if proxyAuth is not None and proxyPasswd is not None:
            proxy = urllib.request.ProxyHandler({'http': 'http://{0}:{1}@{2}'.format(proxyAuth,proxyPasswd,proxyUrl)})
            auth = urllib.request.HTTPBasicAuthHandler()
            opener = urllib.request.build_opener(proxy, auth, urllib.request.HTTPHandler)
            urllib.request.install_opener(opener)

        if url:
            site = urllib.request.urlopen(url)
            html_string = site.read()
            site.close()
            print (html_string)

            match = re.findall(r'href=[\'"]?([^\'" >]+)', html_string)
            for n in match:
                n = n.replace('http://','').replace('https://','')
                if n.startswith(url.replace('http://','')):
                    print(n)
                elif n.startswith('/'):
                    print (n)

                # Open our local file for writing
            """local_file = open(file_name, "w" + file_mode)
                #Write to our local file
                local_file.write(f.read())
                local_file.close()"""

    #handle errors
    except urllib.request.HTTPError as e:
        print (e.code)
    except urllib.request.URLError as e:
        print (e.args)

def getgoogleurl(search,siteurl=False):
    if siteurl==False:
        return 'http://www.google.com/search?q='+urllib.request.quote(search)+'&oq='+urllib.request.quote(search)
    else:
        return 'http://www.google.com/search?q=site:'+urllib.request.quote(siteurl)+'%20'+urllib.request.quote(search)+'&oq=site:'+urllib2.quote(siteurl)+'%20'+urllib2.quote(search)

def getgooglelinks(search,siteurl=False,proxyUrl=None,proxyAuth=None,proxyPasswd=None):
   #google devolvera 403 si no se envia un user-agent valido
   headers = {'User-agent':'Mozilla/11.0'}
   if proxyAuth is not None and proxyPasswd is not None:
       proxy = urllib.request.ProxyHandler({'http': 'http://{0}:{1}@{2}'.format(proxyAuth,proxyPasswd,proxyUrl)})
       auth = urllib.request.HTTPBasicAuthHandler()
       opener = urllib.request.build_opener(proxy, auth, urllib.request.HTTPHandler)
       urllib.request.install_opener(opener)

   req = urllib.request.Request(getgoogleurl(search,siteurl),None,headers)
   site = urllib.request.urlopen(req)
   data = site.read()
   site.close()

   #no beatifulsoup because google html is generated with javascript
   start = data.find('<div id="res">')
   end = data.find('<div id="foot">')
   if data[start:end]=='':
      #error, no links to find
      return False
   else:
      links =[]
      data = data[start:end]
      start = 0
      end = 0
      while start>-1 and end>-1:
          #get only results of the provided site
          if siteurl==False:
            start = data.find('<a href="/url?q=')
          else:
            start = data.find('<a href="/url?q='+str(siteurl))
          data = data[start+len('<a href="/url?q='):]
          end = data.find('&amp;sa=U&amp;ei=')
          if start>-1 and end>-1:
              link =  urllib2.unquote(data[0:end])
              data = data[end:len(data)]
              if link.find('http')==0:
                  links.append(link)
      return links

if __name__ == "__main__":
    print ('''copyright {0}'''.format( __copyright__))
    print ('''license {0}'''.format( __license__))
    print ('''version {0}'''.format( __version__))
    if len(sys.argv) < 2:
        sys.stderr.write("for help use -h o --help")
    elif sys.argv[1]=='-h' or sys.argv[1]=='--help':
        print (b'''
        Puede cargar un texto read Structured Text que siempre debe tener una extension .rst :
        pagina = rst4html('rst_text.rst')
        posteriormente recuperara los datos html creados con :
        pagina.get()
        ''')





#!/bin/env python
# -*- coding: utf-8 -*-
#Author: Pierre Ratinaud
#Copyright (c) 2008-2009, Pierre Ratinaud
#Lisense: GNU/GPL

import urllib2
import socket
import wx
   
def NewVersion(parent):
    version = parent.version.split(' ')
    if len(version) == 3:
        versionnb = float(version[0])
        versionsub = int(version[2])
    else:
        versionnb = float(version[0])
        versionsub = False
    erreur = False
    new = False
    req = urllib2.Request("http://www.iramuteq.org/current_version")
    try:
        LastVersion = urllib2.urlopen(req,'',3)
        lastversion = LastVersion.readlines()
        lastversion = lastversion[0].replace('\n', '').split('-')
        if len(lastversion) == 2 :
            if (float(lastversion[0]) > versionnb) :
                new = True
            elif float(lastversion[0]) == versionnb and versionsub :
                if versionsub < int(lastversion[1].replace('alpha', '')):
                    new = True
        elif len(lastversion) == 1 :
            if (float(lastversion[0]) >= versionnb) and (versionsub) :
                new = True
            elif (float(lastversion[0]) > versionnb) and not versionsub :
                new = True
    except :
        erreur = u"la page n'est pas accessible"
    if not erreur and new :
        msg = u"""
Une nouvelle version d'IRaMuTeQ (%s) est disponible.
Vous pouvez la télécharger à partir du site web iramuteq :
http://www.iramuteq.org""" % '-'.join(lastversion)
        dlg = wx.MessageDialog(parent, msg, u"Nouvelle version disponible", wx.OK | wx.NO_DEFAULT | wx.ICON_WARNING)
        dlg.CenterOnParent()
        if dlg.ShowModal() in [wx.ID_NO, wx.ID_CANCEL]:
             evt.Veto()

#print NewVersion('0.1-alpha18')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of Pymetrick.  The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.

"""Modulo para generar paginas html5 desde ficheros rST (reStructuredText)"""

try:
    from pymetrick.version import __author__, __version__, __license__, __copyright__
except ImportError:
    raise ImportError(
        'Failed to find (autogenerated) version.py '
    )

__date__ = '2013-06-16'
__credits__ = ''
__text__ = 'Generador de codigo HTML5 a partir de ficheros rST'
__file__ = 'rst4html.py'

#--- CHANGES ------------------------------------------------------------------
# 2012-09-21 v0.01 PL: - First version
# 2018-02-20 v0.43 PL: - Upgrade to Python 3.6


import string
import sys
import os
import re
import logging
import logging.handlers
from pymetrick.template import *
import datetime
import time

# Version PY
PY2 = True if sys.version[:1] == '2' else False;
PY3 = True if sys.version[:1] == '3' else False;

labels = {
      'author': b'Autor',
      'authors': b'Autores',
      'organization': b'Organizaci\u00f3n',
      'address': b'Direcci\u00f3n',
      'contact': b'Contacto',
      'version': b'Versi\u00f3n',
      'revision': b'Revisi\u00f3n',
      'status': b'Estado',
      'date': b'Fecha',
      'copyright': b'Copyright',
      'dedication': b'Dedicatoria',
      'abstract': b'Resumen',
      'attention': b'\u00a1Atenci\u00f3n!',
      'caution': b'\u00a1Precauci\u00f3n!',
      'danger': b'\u00a1PELIGRO!',
      'error': b'Error',
      'hint': b'Sugerencia',
      'important': b'Importante',
      'note': b'Nota',
      'tip': b'Consejo',
      'warning': b'Advertencia',
      'contents': b'Contenido'}

''' length_units = [
    em (ems, the height of the element's font)
    ex (x-height, the height of the letter "x")
    px (pixels, relative to the canvas resolution)
    in (inches; 1in=2.54cm)
    cm (centimeters; 1cm=10mm)
    mm (millimeters)
    pt (points; 1pt=1/72in)
    pc (picas; 1pc=12pt)
'''

'''
DEBUG - debug message
INFO - info message
WARNING - warn message
ERROR - error message
CRITICAL - critical message
'''
LOG_LEVEL_VALUE = {'DEBUG':10,'INFO':20,'WARNING':30,'ERROR':40,'CRITICAL':50}
if str(os.environ.get('PYMETRICK_LOG_LEVEL',None)).upper() in LOG_LEVEL_VALUE:
    LOG_LEVEL = LOG_LEVEL_VALUE[str(os.environ.get('PYMETRICK_LOG_LEVEL')).upper()]
else:
    LOG_LEVEL = 30
LOG_FILENAME = '-'.join([os.path.abspath(__file__).split(os.sep)[len(os.path.abspath(__file__).split(os.sep))-1],])[:-3]
LOG = logging.getLogger(LOG_FILENAME)

if 'LD_LIBRARY_PATH' in list(os.environ.keys()):
    # CGI environment
    sys.stdout = sys.stderr
    logging.basicConfig(stream = sys.stderr, level=LOG_LEVEL, format='%(filename)s[line:%(lineno)d] - %(levelname)s - %(message)s')
else:
    # not CGI environment
    logging.basicConfig(stream=sys.stderr)
    hdlr = logging.handlers.RotatingFileHandler(filename=LOG_FILENAME+'.log',mode='a', encoding='utf-8', maxBytes=1048576, backupCount=3)
    formatter = logging.Formatter('%(asctime)s - %(filename)s[line:%(lineno)d] - %(levelname)s - %(message)s')
    hdlr.setFormatter(formatter)
    LOG.addHandler(hdlr)
    LOG.setLevel(LOG_LEVEL)


acceptable_title_elements = ['-','`',':',"'",'"','~','^','_','*','+','#','<','>']

class rst2html(object):
    """Clase para generar objetos HTML5 desde rST reStructuredText"""

    def __init__( self,rst_file ):
        ''' Inicializa la clase rst4html'''
        self._text = []
        self.html = []
        self.title = []
        self._stack = []
        self._stack_reverse = []
        self._deep = 0
        self._empty = 0
        self._date = ''
        self._time = ''
        self._elements = list()
        self._listType = False
        _file = ''
        linea = ''
        f = ''
        try:
            _file = rst_file.lower()
            if _file and _file[-3:]=='rst' and os.path.isfile(_file):
                f = open(_file,"r")
                while True:
                    '''Procesar fichero'''
                    linea = self.readFile(f)
                    #transformar caracteres especiales a HTML
                    linea = template.char2Html(linea)
                    if linea.__len__()==0:
                        '''No existen datos y finaliza el proceso si supera 2 lineas en blanco'''
                        self._empty += 1
                        '''Cada vez que se encuentra una linea en blanco, se incorporan a html todas las lineas leidas'''
                        if self._text.__len__():
                            self.html.append('<p>%s</p>' % ''.join(self._text))
                            self._text = []

                        if self._empty > 1:
                            self._empty = 0
                            break
                        else:
                            self.html.append('<br><br>')
                            linea = ''
                    else:
                        '''Linea contiene datos, inicializar switch de lineas a 0'''
                        self._empty = 0

                        '''Sustituir fecha y hora en un texto'''
                        if re.search(r'^\.{2}?date:{2}',linea.replace(' ','')):
                            '''This document was generated on |date| at |time|.
                               .. date::
                               .. time::

                               A partir de la aparicion de esta directiva date y/o time, se pueden sustituir sus valores en el texto cuando figuren como |date| o |time|
                               Si no se hubiera incorporado ningun valor, el texto a incorporar sera el de la fecha y hora actual
                            '''
                            linea = linea.replace('..','').replace('date::','').strip()
                            if linea.strip().__len__()>0:
                                self._date = linea
                            else:
                                self._date = datetime.date.today().strftime("%d/%m/%Y")
                            linea = self.readFile(f)

                        if re.search(r'^\.{2}?time:{2}',linea.replace(' ','')):
                            linea = linea.replace('..','').replace('time::','').strip()
                            if linea.strip().__len__()>0:
                                self._time = linea
                            else:
                                self._time = time.strftime("%H:%M:%S")
                            linea = self.readFile(f)
                        if re.search(r'|date|',linea.replace(' ','')) and self._date.__len__()>0:
                            linea = linea.replace('|date|',self._date)
                        if re.search(r'|time|',linea.replace(' ','')) and self._time.__len__()>0:
                            linea = linea.replace('|time|',self._time)

                    if not re.search(r'^\.{2}include\:{2}',linea.replace(' ','')):
                        '''Buscar meta'''
                        if re.search(r'^..meta::',linea.replace(' ','')):
                            '''.. meta::
                               :description: The reStructuredText plaintext markup language
                               :keywords: plaintext, markup language

                               <meta name="description"
                               content="The reStructuredText plaintext markup language">
                               <meta name="keywords" content="plaintext, markup language">

                               .. meta::
                               :description lang=en: An amusing story
                               :description lang=fr: Une histoire amusante

                               <meta name="description" lang="en" content="An amusing story">
                               <meta name="description" lang="fr" content="Une histoire amusante">
                            '''
                            pass

                            '''Buscar header'''
                        elif re.search(r'^\.{2}header\:{2}',linea.replace(' ','')):
                            '''.. header:: space for rent

                            Se sustituye en html por :
                            <div class="header">
                               space for rent
                               <hr class="header"/>
                            </div>
                            <div class="document">
                            '''
                            linea = linea.replace('..','').replace('header::','').strip()
                            self.html.append('<div class="header">%s<hr class="header" /></div><div class="document">' % linea )
                            self._stack.append('</div>')
                            self._deep += 1
                            linea = ''

                            '''Buscar footer'''
                        elif re.search(r'^\.{2}footer\:{2}',linea.replace(' ','')):
                            '''.. footer:: space for rent
                            <div class="footer">
                               <hr class="footer" />
                               space for rent
                            </div>
                            '''
                            '''Si existe parrafo almacenado, incorporar antes'''
                            if self._text.__len__():
                                self.html.append('<p>%s</p>' % ''.join(self._text))

                            linea = linea.replace('..','').replace('footer::','').strip()
                            linea = '<div class="footer"><hr class="footer" />%s</div>' % linea
                            '''Antes de indicar los datos de footer comprobar pila stack'''
                            if self._stack.__len__():
                                self._stack_reverse = self._stack
                                self._stack_reverse.reverse()
                                for i in range(self._stack_reverse.__len__()):
                                    self.html.append(self._stack_reverse.pop())
                                    self._deep -= 1
                            self.html.append(linea)
                            self._stack = []
                            self._text = []
                            linea = ''

                            '''Buscar cualquier caracter repetido 5 veces o mas de forma continua en una linea sera un TITULO o encabezado de SECCION'''
                        elif re.search(r'^(\-|\`|\:|\'|\"|\~|\^|\_|\*|\+|\#|\<|\>){5,}',linea.replace(' ','')):

                            if linea.replace(' ','')[1:1] not in self.title:
                                self.title.append(linea.replace(' ','')[0:1])
                            for i in range(self.title.__len__()):
                                if self.title[i]==linea.replace(' ','')[0:1]:
                                    '''Recuperar linea anterior donde se encuentra el titulo'''
                                    if self._text.__len__():
                                        linea = self._text.pop()
                                    '''Si existe parrafo almacenado, incorporar antes'''
                                    if self._text.__len__():
                                        self.html.append('<p>%s</p>' % ''.join(self._text))
                                    '''Comprobar grado de identacion'''
                                    if i<self._deep:
                                        if self._stack.__len__():
                                            self._stack_reverse = self._stack
                                            self._stack_reverse.reverse()
                                            self._text.append(self._stack_reverse.pop())
                                            self._stack = self._stack_reverse
                                            self._stack.reverse()
                                            self._deep -= 1

                                    '''Crear TITULO o SECCION en html'''
                                    self.html.append('<div><h%d class="title">%s</h%d>' % (i+1,linea,i+1))
                                    self._stack.append('</div>')
                                    self._deep += 1
                            linea = ''
                            self._text = []


                            '''Buscar cualquier caracter 1 vez es una LISTA'''
                        elif re.search(r'^([a-zA-Z0-9]{1,2}|\-{1}|\+{1}|\*{1})(\)|\.){1}\s',linea):
                            '''Linea que comienza con un numero o letra, seguido de un punto "." o
                               de un parentesis ")"  sera reconocido como lista.
                               <OL> o <UL> se incluira para numeros o letras respectivamente
                               -. guio
                               n para lista
                               +. signo mas para lista
                               *. asterisco para lista
                               1. numeros
                               A. letra mayuscula
                               a. letra misnuscula
                               1) numero seguido de un parentesis
                            '''
                            '''Si existe parrafo almacenado, incorporar antes'''
                            if self._text.__len__():
                                self.html.append('<p>%s</p>' % ''.join(self._text))
                            self._elements = list()
                            # El indicador de lista, por defecto sera un punto
                            if re.search(r'^[0-9]{1,2}(\)|\.){1}\s',linea):
                                # excepto si el primer elemento de lista esta indicado con un numero ordinal
                                self._listType = True
                            else:
                                self._listType = False
                            # Eliminar codigo de lista y seleccionar solo item de lista                            repl = re.findall(r'^([a-zA-Z0-9]{1,2}|\-{1}|\+{1}|\*{1})(\)|\.|\S|\s)',linea)
                            repl = re.findall(r'^([a-zA-Z0-9]{1,2}|\-{1}|\+{1}|\*{1})(\)|\.){1}\s',linea)
                            self._elements.append(linea.replace(''.join(repl[0]),'').strip())
                            while linea.__len__()>0:
                                pos = f.tell()
                                linea = self.readFile(f)
                                if re.search(r'^(\w|\-|\+|\*){1}(\)|\.){1}',linea.replace(' ','')):
                                    # Eliminar codigo de lista y seleccionar solo item de lista
                                    repl = re.findall(r'^([a-zA-Z0-9]{1,2}|\-{1}|\+{1}|\*{1})(\)|\.){1}\s',linea)
                                    self._elements.append(linea.replace(''.join(repl[0]),'').strip())
                                else:
                                    f.seek(pos)
                                    break
                            self.listElement(self._listType,*self._elements)
                            self._elements = []
                            linea = ''
                            self._text = []

                            '''Buscar imagen'''
                        elif re.search(r'^\.{2}?image:{2}',linea.replace(' ','')):
                            '''.. image:: images/biohazard.png
                               :height: 100px
                               :width: 200px
                               :alt: alternate text

                               <img src="images/biohazard.png" alt="biohazard" height="42" width="42">
                            '''
                            token = {'image':'','height':'','width':'','alt':''}
                            token['image']= linea.replace('..','').replace('image::','').strip()
                            while linea.__len__()>0:
                                pos = f.tell()
                                linea = self.readFile(f)
                                if re.search(r'^\:{1}(.+?)\:{1}',linea):
                                    for i in list(token.keys()):
                                        if (':%s:' % i) in linea:
                                            token[i] = linea.replace(':%s:' % i,'').strip()
                                            break
                                else:
                                    f.seek(pos)
                                    break
                            '''Crear imagen en html'''
                            self._text.append('<img src="%s" alt="%s" height="%s" width="%s">' % (token["image"],token["alt"],token["height"],token["width"]))
                            token = {}
                            linea = ''

                            '''Sustituir imagen en un texto'''
                        elif re.search(r'^\.{2}?\|(.+?)\|?image:{2}',linea.replace(' ','')):
                            '''The |biohazard| symbol must be used on containers used to dispose of medical waste.
                               .. |biohazard| image:: biohazard.png
                            '''
                            replace_text = []
                            token = {'image':'','height':'','width':'','alt':''}
                            '''Buscar valor/es a reemplazar en el texto'''
                            replace_text = re.findall(r'\|(.+?)\|',linea)
                            for i in replace_text:
                                linea = linea.replace(('|%s|' % i),'')
                            '''Buscar link de imagen solo'''
                            token['image']= linea.replace('..','').replace('image::','').strip()
                            while linea.__len__()>0:
                                pos = f.tell()
                                linea = self.readFile(f)
                                if re.search(r'^\:{1}(.+?)\:{1}',linea):
                                    for i in list(token.keys()):
                                        if (':%s:' % i) in linea:
                                            token[i] = linea.replace(':%s:' % i,'').strip()
                                            break
                                else:
                                    f.seek(pos)
                                    break
                            linea = ('<figure><img src="%s" alt="%s" height="%s" width="%s"><figcaption><p>%s</p></figcaption></figure>' % (token["image"],token["alt"],token["height"],token["width"],token["alt"]))
                            if self._text.__len__():
                                '''Reemplaza los valores del texto por el link a la imagen'''
                                for j in replace_text:
                                    self._text = [i.replace( '|%s|' % j ,linea)  for i in self._text]
                            token = {}
                            replace_text = []
                            linea = ''


                            '''Incorporar codigo de programacion'''
                        elif re.search(r'^\.{2}?code-block:{2}',linea.replace(' ','')):
                            '''.. code-block::
                               def my_function():
                                   "just a test"
                                   print 8/2
                               ::

                               El codigo finalizara con :: en la ultima linea.

                               En el codigo, la libreria sustituira < por $lt; y > por &gt; para no confundirse con la
                               codificacion HTML.

                               Generando el siguiente codigo:
                               <code>   ...codigo ... </code>

                            '''
                            replace_text = []
                            '''La primera linea que identifica code-block no se utilizara'''
                            replace_text.append('<br><br><code display="block">')
                            while linea.__len__()>0:
                                '''La linea se debe leer sin tratamiento'''
                                linea = f.readline().rstrip()
                                if re.search(r'^\:{2}',linea) and linea.strip().__len__()==2:
                                    'Es el final del codigo'
                                    replace_text.append('</code><br><br>')
                                    break
                                else:
                                    replace_text.append(linea.replace('<','&lt;').replace('>','&gt;').replace('"','&quot;').replace(' ','&nbsp;')+'<br>')
                            self._text.append(''.join(replace_text))

                            '''Incorporar codificacion matematica'''
                        elif re.search(r'^\.{2}?math:{2}',linea.replace(' ','')):
                            '''.. math::

                               ::
                               Se emplea la notación MathML  (http://www.w3.org/TR/MathML)

                               La expresion se sustituira por <math display="block"> ...codigo matematico ... </math>
                            '''
                            replace_text = []
                            '''La primera linea que identifica code-block no se utilizara'''
                            replace_text.append('<br><br><math display="block">')
                            while linea.__len__()>0:
                                '''La linea se debe leer sin tratamiento'''
                                linea = f.readline().strip()
                                if re.search(r'^\:{2}',linea) and linea.strip().__len__()==2:
                                    'Es el final del codigo'
                                    replace_text.append('</math><br><br>')
                                    break
                                else:
                                    replace_text.append(linea)
                            self._text.append(''.join(replace_text))

                            '''Incorporar tablas'''
                        elif re.search(r'^\.{2}?table:{2}',linea.replace(' ','')):
                            '''.. table:: Truth table for "not"
                               =====  =====
                                 A    not A
                               =====  =====
                               False  True
                               True   False
                               =====  =====
                               ::


                               La expresion se sustituira por
                               <table border="1">
                               <colgroup>
                                  <col width="31%"> .. </col>
                                  <col width="31%"> .. </col>
                                  <col width="38%"> .. </col>
                               </colgroup>
                               <thead valign="bottom">
                                  <tr> .. </tr>
                                  <tr> .. </tr>
                               </thead>
                               <tbody valign="top">
                                  <tr> .. </tr>
                                  <tr> .. </tr>
                                  <tr> .. </tr>
                                  <tr> .. </tr>
                               </tbody>
                               </table>
                            '''
                            pass

                            '''Incorporar hiperlink'''
                        elif re.search(r'^\.{2}_{1}.*:{1}',linea.replace(' ','')):
                            '''passExternal hyperlinks, like Python_.
                               .. _Python: http://www.python.org/

                               se sustituye _Python en el lugar de Python_
                               <a href="http://www.python.org/">Python</a>

                            '''
                            replace_text = []
                            replace_text = re.findall(r'^\.{2}.*_{1}(.+?):{1}',linea)
                            for i in replace_text:
                                linea = linea.replace(' ','').replace(('.._%s:' % i),'')
                                linea = ('<a href="%s" >%s</a>' % (linea,i))
                                if self._text.__len__():
                                    '''Reemplaza los valores de hiperlink en el texto'''
                                    for j in range(self._text.__len__()):
                                        self._text[j] = self._text[j].replace(("%s_" % i), linea )
                            replace_text = []
                            linea = ''

                            '''Buscar hiperlink anonimos http'''
                        elif re.search(r'http://(.+?)\s+',linea) or re.search(r'https://(.+?)\s+',linea):
                            '''External hyperlinks anonimos
                               se sustituye http://www.python.org/ por
                               <a href="http://www.python.org/">http://www.python.org/</a>
                            '''
                            replace_text = []
                            replace_text = re.findall(r'http://(.+?)\s+',linea)
                            for i in replace_text:
                                linea = linea.replace(('http://%s' % i),('<a href="http://%s" >http://%s</a>' % (i,i)))
                            replace_text = []
                            replace_text = re.findall(r'https://(.+?)\s+',linea)
                            for i in replace_text:
                                linea = linea.replace(('https://%s' % i),('<a href="https://%s" >https://%s</a>' % (i,i)))
                            replace_text = []

                        '''inserta una linea en blanco'''
                        if re.search(r'^\:{2}',linea) and linea.strip().__len__()==2:
                            '''::
                               Identifica una linea de salto
                            '''
                            '''Si linea en blanco o salto, se incorporan a html todas las lineas leidas como parrafo'''
                            if self._text.__len__():
                                self.html.append('<p>%s</p>' % ''.join(self._text))
                                self._text = []
                            self.html.append('<br>')
                            linea = ''

                        '''inserta una linea de separacion'''
                        if re.search(r'^\_{3}',linea) and linea.strip().__len__()==3:
                            '''___
                               Inserta una linea <hr>
                            '''
                            '''Si linea de separacion, se incorporan a html todas las lineas leidas como parrafo'''
                            if self._text.__len__():
                                self.html.append('<p>%s</p>' % ''.join(self._text))
                                self._text = []
                            self.html.append('<hr>')
                            linea = ''

                        '''otras directivas no soportadas se anulan'''
                        if re.search(r'^\.{2}(.+?):{2}',linea.replace(' ','')):
                            '''..   ::
                            '''
                            linea = ''

                        '''texto con formato bold'''
                        if re.search(r'\*{2}.*\*{2}',linea.replace(' ','')):
                            replace_text = []
                            replace_text = re.findall(r'\*{2}.*\*{2}',linea)
                            if replace_text:
                                for i in replace_text:
                                    linea = linea.replace( i ,'<b>%s</b>' % i.replace('*',''))

                        '''texto con formato italic'''
                        if re.search(r'\*{1}.*\*{1}',linea.replace(' ','')):
                            replace_text = []
                            replace_text = re.findall(r'\*{1}.*\*{1}',linea)
                            if replace_text:
                                for i in replace_text:
                                    linea = linea.replace( i ,'<i>%s</i>' % i.replace('*',''))


                        '''Se carga el texto que no es de control'''
                        if linea.__len__()>0:
                            '''Guarda informacion hasta realizar tratamiento'''
                            self._text.append(' '+linea)

                        linea = ''


                    else:
                        ''' Buscar informacion literal'''
                        if re.search(r'^..include::',linea.replace(' ','')):
                            pass
            else:
                '''No existe fichero rST'''
                _file = ''

            '''Antes de salir comprobar pila de stack'''
            if self._stack.__len__():
                self._stack_reverse = self._stack
                self._stack_reverse.reverse()
                for i in range(self._stack_reverse.__len__()):
                    self._text.append(self._stack_reverse.pop())
                    self._deep -= 1
                self._stack = []

        except Exception as e:
            tb = sys.exc_info()[2]
            print('Error <%s> en linea %s !!!' % (str(e),tb.tb_lineno) )
        finally:
            if _file and os.path.isfile(_file):
                f.close()

    def get(self,withHtml=False):
        '''Recupera el contenido HTML generado desde el fichero leido'''
        if withHtml:
            # Puede devolver una página HTML totalmente funcional
            self.html = '<!DOCTYPE html><html><head></head><body>'+'\n'.join(self.html)+'</body></html>'
        else:
            # o codigo HTML para incluir en una pagina HTML
            self.html = '\n'.join(self.html)
        return self.html

    def readFile( self , f):
        linea = ' '+f.readline()
        # Pueden aparecer en la linea Byte Order Mark o marca en little-endian \xEF\xBB\xBF que debemos eliminar
        linea = linea.replace('\xef\xbb\xbf','')
        return linea.strip()

    def listElement(self, withNumber=False, *items):
        '''Creara una lista de numeros o puntos a partir de una lista de elementos'''
        if len(items):
            if withNumber:
                # lista con numeros
                self.html.append('<ol>')
            else:
                # lista con puntos
                self.html.append('<ul>')
            for item in items:
                self.html.append('<li>%s</li>' % item)
            if withNumber:
                # finaliza lista con numeros
                self.html.append('</ol>')
            else:
                # finaliza lista con puntos
                self.html.append('</ul>')

    def headingElement(self, content):

        newid = makeID(content, self.ids)
        self.ids.append(newid)

        self.write('<h2 id="%s">' % newid)
        self.write(content)
        self.write('</h2>')
        self.write('\n')

    def paragraphElement(self, content):
        self.write('<p>')
        self.write(content)
        self.write('</p>')
        self.write('\n')



if __name__ == "__main__":
    print ('''copyright {0}'''.format( __copyright__))
    print ('''license {0}'''.format( __license__))
    print ('''version {0}'''.format( __version__))
    if len(sys.argv) < 2:
        sys.stderr.write("for help use -h o --help")
    elif sys.argv[1]=='-h' or sys.argv[1]=='--help':
        print (b'''
        Puede cargar un texto read Structured Text que siempre debe tener una extension .rst :
        pagina = rst4html('rst_text.rst')
        posteriormente recuperara los datos html creados con :
        pagina.get()
        ''')




#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of Pymetrick.  The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.

try:
    from pymetrick.version import __author__, __version__, __license__, __copyright__
except ImportError:
    raise ImportError(
        'Failed to find (autogenerated) version.py '
    )

__date__ = '2023-04-22'
__credits__ = ''
__text__ = 'Transformar texto en voz'
__file__ = 'speech.py'

#--- CHANGES ------------------------------------------------------------------
# 2023-04-22 v0.01 PL: - First version

import os, sys
# pip install pyttsx3
from   platform import system
import pyttsx3
import logging
import logging.handlers

'''
DEBUG - debug message
INFO - info message
WARNING - warn message
ERROR - error message
CRITICAL - critical message
'''
LOG_LEVEL_VALUE = {'DEBUG':10,'INFO':20,'WARNING':30,'ERROR':40,'CRITICAL':50}
if str(os.environ.get('PYMETRICK_LOG_LEVEL',None)).upper() in LOG_LEVEL_VALUE:
    LOG_LEVEL = LOG_LEVEL_VALUE[str(os.environ.get('PYMETRICK_LOG_LEVEL')).upper()]
else:
    LOG_LEVEL = 30
LOG_FILENAME = '-'.join([os.path.abspath(__file__).split(os.sep)[len(os.path.abspath(__file__).split(os.sep))-1],])[:-3]
LOG = logging.getLogger(LOG_FILENAME)

if 'LD_LIBRARY_PATH' in list(os.environ.keys()):
    # CGI environment
    sys.stdout = sys.stderr
    logging.basicConfig(stream = sys.stderr, level=LOG_LEVEL, format='%(filename)s[line:%(lineno)d] - %(levelname)s - %(message)s')
else:
    # not CGI environment
    logging.basicConfig(stream=sys.stderr)
    hdlr = logging.handlers.RotatingFileHandler(filename=LOG_FILENAME+'.log',mode='a', encoding='utf-8', maxBytes=1048576, backupCount=3)
    formatter = logging.Formatter('%(asctime)s - %(filename)s[line:%(lineno)d] - %(levelname)s - %(message)s')
    hdlr.setFormatter(formatter)
    LOG.addHandler(hdlr)
    LOG.setLevel(LOG_LEVEL)
    
class Speech(object):

    def __init__(self, **kwargs):

        try:
        
            self.thisOS = system() # averiguar Sistema operativo actual
            self.engine = pyttsx3.init()    # inicializar motor de voz
            if 'rate' in kwargs:
                self.engine.setProperty('rate', kwargs['rate'])
            else:
                self.engine.setProperty('rate'  ,130)   # Aumentar velocidad 30%
            if 'voice' in kwargs:
                self.engine.setProperty('voice', kwargs['voice'])
            else:
                if self.thisOS == 'Linux':
                    self.engine.setProperty('voice', 'spanish-latin-am+f5') #ASIGNAR Español latino
                    # agregar +f5 para simular voz femenina (1-5)
                if self.thisOS == 'Windows':
                    voices = engine.getProperty('voices')       #getting details of current voice
                    self.engine.setProperty('voice', voices[0].id)  #changing index, changes voices. o for male
                    #self.engine.setProperty('voice', voices[1].id)   #changing index, changes voices. 1 for female

                    #self.engine.setProperty('voice',f'''HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Speech\Voices\Tokens\{voices[0]}''')
            if 'volume' in kwargs:
                self.engine.setProperty('volume', kwargs['volume'])
            else:   
                self.engine.setProperty('volume',1.0)   # Poner el volumen al 100%: Minimo:0, Maximo: 1
        except Exception as e:
            tb = sys.exc_info()[2]
            LOG.error('<%s> in line %s !!!' % (str(e),tb.tb_lineno))


    def say(self,text=None):
        try:
            if text is not None:
                self.engine.say(text)
                self.engine.runAndWait()                 # Esperar a que termine de hablar
                self.engine.stop()                       # Detener motor tts            
        except Exception as e:
            tb = sys.exc_info()[2]
            LOG.error('<%s> in line %s !!!' % (str(e),tb.tb_lineno))            

if __name__ == '__main__' :
    print ('''copyright {0}'''.format( __copyright__))
    print ('''license {0}'''.format( __license__))
    print ('''version {0}'''.format( __version__))
    if len(sys.argv) < 2:
        sys.stderr.write("for help use -h o --help")
    elif sys.argv[1]=='-h' or sys.argv[1]=='--help':
        print (b'''Speech : Transformar texto en voz\n''')
    



#!/usr/bin/python
# -*- coding: utf-8 *-*

"""Modulo para gestionar datos en SQLite"""

__author__ = 'Fco.Javier Tamarit V <javtamvi@gmail.com>'
__copyright__ = "Copyright � 2012 Betaksistemas"
__date__ = '2012-09-21'
__version__ = '0.01'
__license__ = 'GPLv3'
__credits__ = ''
__text__ = 'Instalador local de los módulos de PYMETRICK en plataformas windows y linux'
__file__ = 'autosetup.py'

#--- CHANGES ------------------------------------------------------------------
# 2012-09-21 v0.01 PL: - First version

import os
import sys

platform = sys.platform
# Nos entrega la version de python en formato de dict
version = sys.version_info
# Nos entrega el path en forma de list
path = sys.path
d = ''
d_list = list()
for p in path:
    if p.find('site-packages')>0:
        d = p[:p.find('site-packages')+len('site-packages')]
    if p.find('dist-packages')>0:
        d = p[:p.find('dist-packages')+len('dist-packages')]
    # Guarda todos los directorios que contienen site-packages o dist-packages y no repetidos
    if len(d)>0:
        if d not in d_list:
            d_list.append(d)
# Comienza instalacion de paquete
for d in d_list:
    d = d+os.sep
    d = d+'pymetrick'
    if not os.path.isdir(d):
        os.mkdir(d)
    # directorio origen
    source = os.listdir(os.getcwd())
    if platform.startswith('linux'):
        for f in source:
            os.system("cp -f "+os.getcwd()+os.sep+f+" "+d)
            print os.getcwd()+os.sep+f+" "+d
    if platform.startswith('win'):
        for f in source:
            os.system("copy "+os.getcwd()+os.sep+f+" "+d)



#-------------------------------------------------------------------------------
# Name:        Queue
# Purpose:
#
# Author:      JAVIER
#
# Created:     02/03/2018
# Copyright:   (c) JAVIER 2018
# Licence:     GPLv3
#-------------------------------------------------------------------------------
import sys
# sin bytecode
sys.dont_write_bytecode = True

class Queue:
    def __init__(self):
        self.items = []
        self.frontIdx = 0

    def __compress(self):
        newitems = []
        for i in range(self.frontIdx,len(self.items)):
            newitems.append(self.items[i])

        self.items = newitems
        self.frontIdx = 0

    def dequeue(self):
        if self.isEmpty():
            raise RuntimeError("Attempt to dequeue an empty queue")

        # When queue is half full, compress it. This
        # achieves an amortized complexity of O(1) while
        # not letting the list continue to grow unchecked.
        if self.frontIdx * 2 > len(self.items):
            self.__compress()

        item = self.items[self.frontIdx]
        self.frontIdx += 1
        return item

    def enqueue(self,item):
        self.items.append(item)

    def front(self):
        if self.isEmpty():
            raise RuntimeError("Attempt to access front of empty queue")

        return self.items[self.frontIdx]


    def isEmpty(self):
        return self.frontIdx == len(self.items)

    def clear(self):
        self.items = []
        self.frontIdx = 0

def main():
    q = Queue()
    items = list(range(10))
    items2 = []

    for k in items:
        q.enqueue(k)

    if q.front() == 0:
        print("Test 1 Passed")
    else:
        print("Test 1 Failed")

    while not q.isEmpty():
        items2.append(q.dequeue())

    if items2 != items:
        print("Test 2 Failed")
    else:
        print("Test 2 Passed")

    for k in items:
        q.enqueue(k)

    items2 = []

    while not q.isEmpty():
        items2.append(q.dequeue())

    if items2 != items:
        print("Test 3 Failed")
    else:
        print("Test 3 Passed")

    try:
        q.dequeue()
        print("Test 4 Failed")

    except RuntimeError:
        print("Test 4 Passed")
    except:
        print("Test 4 Failed")

    try:
        q.front()
        print("Test 5 Failed")

    except RuntimeError:
        print("Test 5 Passed")
    except:
        print("Test 5 Failed")

if __name__=="__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of Pymetrick.  The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.

"""Modulo de gestion de rutas"""

try:
    from pymetrick.version import __author__, __version__, __license__, __copyright__
except ImportError:
    raise ImportError(
        'Failed to find (autogenerated) version.py '
    )

__date__ = '2012-09-21'
__credits__ = ''
__text__ = 'Gestion de rutas'
__file__ = 'routing.py'

#--- CHANGES ------------------------------------------------------------------
# 2012-09-21 v0.01 PL: - First version
# 2017-10-02 v0.43 PL: - Migrar a PYTHON 3.6


"""
    He aquí un ejemplo simple que crea un mapa de URL para una aplicación con
    dos subdominios (www y kb) y algunas reglas URL:

    >>> m = Map([
    ...     # Static URLs
    ...     ('/', endpoint='static/index'),
    ...     ('/about', endpoint='static/about'),
    ...     ('/help', endpoint='static/hcd
    ...     Subdomain('kb', [
    ...         Rule('/', endpoint='kb/index'),
    ...         Rule('/browse/', endpoint='kb/browse'),
    ...         Rule('/browse/<int:id>/', endpoint='kb/browse'),
    ...         Rule('/browse/<int:id>/<int:page>', endpoint='kb/browse')
    ...     ])
    ... ], default_subdomain='www')



"""
import sys
# sin bytecode
sys.dont_write_bytecode = True
import os
import re
import logging
import logging.handlers

# PY3
long = int

'''
DEBUG - debug message
INFO - info message
WARNING - warn message
ERROR - error message
CRITICAL - critical message
'''
if str(os.environ.get('PYMETRICK_LOG_LEVEL',None)).upper() in ('DEBUG','INFO','WARNING','ERROR','CRITICAL'):
    LOG_LEVEL = eval('.'.join(['logging',str(os.environ.get('PYMETRICK_LOG_LEVEL')).upper()]))
else:
    LOG_LEVEL = eval('logging.WARNING')
LOG_FILENAME = '-'.join([os.path.abspath(__file__).split(os.sep)[len(os.path.abspath(__file__).split(os.sep))-1],])[:-3]
LOG = logging.getLogger(LOG_FILENAME)

if 'LD_LIBRARY_PATH' in list(os.environ.keys()):
    # CGI environment
    sys.stdout = sys.stderr
    logging.basicConfig(stream = sys.stderr, level=LOG_LEVEL, format='%(filename)s[line:%(lineno)d] - %(levelname)s - %(message)s')
else:
    # not CGI environment
    logging.basicConfig(stream=sys.stderr)
    hdlr = logging.handlers.RotatingFileHandler(filename=LOG_FILENAME+'.log',mode='a', encoding='utf-8', maxBytes=1048576, backupCount=3)
    formatter = logging.Formatter('%(asctime)s - %(filename)s[line:%(lineno)d] - %(levelname)s - %(message)s')
    hdlr.setFormatter(formatter)
    LOG.addHandler(hdlr)
    LOG.setLevel(LOG_LEVEL)

class Map(object):
    """Map relaciona las rutas con los controladores a ejecutar.
    existen dos parametros esenciales :
    default_subdomain : Dominio por defecto www
    redirect_default  : Cuando no existe la ruta solicitada, se redirige a una ruta por defecto.
    url_map = Map([
                ('/', subdomain='{username}', endpoint='user/homepage'),
                ('/stats', subdomain='{username}', endpoint='user/stats')
            ])
    Para incorporar nuevas rutas utiliza add, si una ruta ya existiera, se sustituira por la incorporada.
    add('/ideas/{idea}', controller = 'idea')
    add('/users/{usuario}', controller = 'user')
    add('/tags/{etiquetas}', controller = 'tag')

    _rules  es el conjunto de rutas de validacion, cada elemento de _rules se compone de una ruta y un controlador
            Cada ruta puede ser estatica p.e. : /login o dinámica p.e. : /articulos/{año}/{dia}
    """

    def __init__(self, default_subdomain='', redirect_defaults=True):
        try:
            self._rules = {}
            self.default_subdomain = default_subdomain
            self.redirect_defaults = redirect_defaults
        except Exception as e:
            tb = sys.exc_info()[2]
            LOG.error('Error en add <%s> en linea %s !!!' % (repr(e),tb.tb_lineno))


    def add(self, rule='', controller=''):
        """Incorporar una nueva ruta dirigiendo la accion a un controlador
        _rules  reglas de validacion

        """
        try:
            if rule and controller:
                """Incorporar nueva regla o sustituir si existe"""
                self._rules[rule]=controller
        except Exception as e:
            tb = sys.exc_info()[2]
            LOG.error('Error en add <%s> en linea %s !!!' % (repr(e),tb.tb_lineno))

    def match(self, path):
        """Compruebe si la regla coincide con una ruta determinada"""

        _match = []
        _match_number = 0
        # Comprobar si la ruta contiene datos del tipo ?nombre_variable=dato y extrae solo informacion de ruta
        if re.search(r'\?',path):
            path = path[:path.find(r"?")]
        # Si es un path dinamico debemos comprobarlo minuciosamente
        _dict_dinamico = dict()
        try:

            _path = path.strip("/").split("/")
            for n,m in list(self._rules.items()):
                _n = n.strip("/").split("/")
                _match_number = 0
                for i in range(0,len(_n)):
                    if len(_path)>=i+1:
                        if _n[i].startswith('{') and _n[i].endswith('}') and len(_path)>=i+1:
                            # Los parametros en la ruta aparecen entre {}
                            # crear un diccionario emparejando parametros con valor
                            _dict_dinamico[_n[i].strip("{}")]=_path[i]
                            _match_number +=1
                        elif _path[i]==_n[i]:
                            # Cuenta coincidencias entre ruta y valor
                            _match_number += 1
                        if len(_path)==_match_number and _match_number>0:
                            if len(_path)==_match_number and len(_n)==_match_number:
                                # Prepara resultado : ruta, controlador, coincidencia
                                if len(_dict_dinamico)>0:
                                    # Crear la funcion con parametros
                                    m = "%s(**%s)" % (m,_dict_dinamico)
                                _match.append((n,m,_match_number))

            if not _match and self.redirect_defaults:
                """404	NOT FOUND. El servidor no halló nada en la dirección especificada"""
                """Devuelve resultado como ruta, controlador y coincidencia"""
                _match.append(('/','404',0))
            """Devuelve solo la funcion a evaluar"""
            return _match[0][1]
        except Exception as e:
            tb = sys.exc_info()[2]
            LOG.error('Error en Map.match <%s> en linea %s !!!' % (repr(e),tb.tb_lineno))

    def __del__(self):
        '''Eliminar objeto y objetos dependientes'''
        LOG.debug("del ha eliminado el objeto")

if __name__ == "__main__":
    print ('''copyright {0}'''.format( __copyright__))
    print ('''license {0}'''.format( __license__))
    print ('''version {0}'''.format( __version__))
    if len(sys.argv) < 2:
        sys.stderr.write("For help use -h o --help")
    elif sys.argv[1]=='-h' or sys.argv[1]=='--help':
        print ('''
        from pymetrick.routing import *

        Primero debe crear una ruta al subdominio de la siguiente forma :

        rules_web = Map(default_subdomain='www',redirect_defaults='/')

        Las rutas a páginas se crean de la siguiente forma :

        rules_web.add(rule='/login',controller='login')
        rules_web.add(rule='/login/nada',controller='login')
        rules_web.add(rule='/login/nada/todo',controller='login')
        rules_web.add(rule='/registro',controller='registro')
        rules_web.add(rule='/prueba',controller='prueba')

        Cuando busque una ruta debe indicarlo como :

        rules_web.match('login')
        rules_web.match('/login/nada/todo')
        rules_web.match('/varios')

        esto ejecutara el controlador parametrizado.

        Si la ruta obtenida del cliente, contiene datos del tipo
        '/auth?user=javier&passw=tonto'
        se eliminaran para evaluar correctamente la ruta

        Y si no existe la ruta, devolvera 404 y podremos redirigirlo
        if rules_web.match('/no_existe')=="404":
           rules_web.match('/')
        ''')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import sys
# sin bytecode
sys.dont_write_bytecode = True
import os
import logging
import logging.handlers
import requests

import firebase_admin
from firebase_admin import auth
from firebase_admin import credentials, firestore
from firebase_admin import storage
from firebase_admin import exceptions
#from google.oauth2 import service_account

'''
DEBUG - debug message
INFO - info message
WARNING - warn message
ERROR - error message
CRITICAL - critical message
'''
LOG_LEVEL_VALUE = {'DEBUG':10,'INFO':20,'WARNING':30,'ERROR':40,'CRITICAL':50}
if str(os.environ.get('PYMETRICK_LOG_LEVEL',None)).upper() in LOG_LEVEL_VALUE:
    LOG_LEVEL = LOG_LEVEL_VALUE[str(os.environ.get('PYMETRICK_LOG_LEVEL')).upper()]
else:
    LOG_LEVEL = 30
LOG_FILENAME = '-'.join([os.path.abspath(__file__).split(os.sep)[len(os.path.abspath(__file__).split(os.sep))-1],])[:-3]
LOG = logging.getLogger(LOG_FILENAME)

if 'LD_LIBRARY_PATH' in list(os.environ.keys()):
    # CGI environment
    sys.stdout = sys.stderr
    logging.basicConfig(stream = sys.stderr, level=LOG_LEVEL, format='%(filename)s[line:%(lineno)d] - %(levelname)s - %(message)s')
else:
    # not CGI environment
    logging.basicConfig(stream=sys.stderr)
    hdlr = logging.handlers.RotatingFileHandler(filename=LOG_FILENAME+'.log',mode='a', encoding='utf-8', maxBytes=1048576, backupCount=3)
    formatter = logging.Formatter('%(asctime)s - %(filename)s[line:%(lineno)d] - %(levelname)s - %(message)s')
    hdlr.setFormatter(formatter)
    LOG.addHandler(hdlr)
    LOG.setLevel(LOG_LEVEL)


_DEFAULT_TIMEOUT = 60

BUCKET_NAME   = '****.appspot.com'
BUCKET_FOLDER = '/b/****.appspot.com/o'
LOCAL_FOLDER  = 'c:/****'
GOOGLE_CERTIFICATE = "certificates/****-firebase.json"

class GStorage(object):
    def __init__(self, *args, **kwargs):
        try:
            self.bucketName = BUCKET_NAME
            self.bucketFolder = BUCKET_FOLDER
            self.localFolder= LOCAL_FOLDER
            cred = credentials.Certificate(GOOGLE_CERTIFICATE)
            firebase_admin.initialize_app(cred, {'storageBucket': self.bucketName})
            self.bucket = storage.bucket() # return google.cloud.storage.Bucket
           
        except exceptions.FirebaseError as e:
            if hasattr(e, 'message'):
                LOG.error('<%s> in line %s !!!' % (e.message,format(sys.exc_info()[-1].tb_lineno)))
            else:
                LOG.error('<%s> in line %s !!!' % (e,format(sys.exc_info()[-1].tb_lineno)))


    def file_upload(self,source_file_name, destination_blob_name):
        """Uploads a file to the bucket."""
        # source_file_name = "local/path/to/file"
        # destination_blob_name = "storage-object-name"
        try:
            blob = self.bucket.blob(destination_blob_name)
            blob.upload_from_filename(source_file_name)
            blob.make_public()
        except exceptions.FirebaseError as e:
            if hasattr(e, 'message'):
                LOG.error('<%s> in line %s !!!' % (e.message,format(sys.exc_info()[-1].tb_lineno)))
            else:
                LOG.error('<%s> in line %s !!!' % (e,format(sys.exc_info()[-1].tb_lineno)))
            
    def file_download(self,source_blob_name, destination_file_name):
        """Downloads a blob from the bucket.
        fix requests >= 2.18.0
        """
        # source_blob_name = "storage-object-name"
        # destination_file_name = "local/path/to/file"
        try:
            blob = self.bucket.blob(source_blob_name)
            blob.download_to_filename(destination_file_name)
        except exceptions.FirebaseError as e:
            if hasattr(e, 'message'):
                LOG.error('<%s> in line %s !!!' % (e.message,format(sys.exc_info()[-1].tb_lineno)))
            else:
                LOG.error('<%s> in line %s !!!' % (e,format(sys.exc_info()[-1].tb_lineno)))

    def file_delete(self,blob_name):
        """Deletes a blob from the bucket."""
        # blob_name = "your-object-name"
        try:
            blob = self.bucket.blob(blob_name)
            blob.delete()
        except exceptions.FirebaseError as e:
            if hasattr(e, 'message'):
                LOG.error('<%s> in line %s !!!' % (e.message,format(sys.exc_info()[-1].tb_lineno)))
            else:
                LOG.error('<%s> in line %s !!!' % (e,format(sys.exc_info()[-1].tb_lineno)))

    def file_rename(self,blob_name, new_name):
        """Renames a blob."""
        # blob_name = "your-object-name"
        # new_name = "new-object-name"
        try:
            blob = self.bucket.blob(blob_name)
            new_blob = self.bucket.rename_blob(blob, new_name)
        except exceptions.FirebaseError as e:
            if hasattr(e, 'message'):
                LOG.error('<%s> in line %s !!!' % (e.message,format(sys.exc_info()[-1].tb_lineno)))
            else:
                LOG.error('<%s> in line %s !!!' % (e,format(sys.exc_info()[-1].tb_lineno)))
        

    def list_blobs(self):
        """Lists all the blobs in the bucket."""
        try:
            fileList = []
            if self.bucket.exists():
                files = self.bucket.list_blobs()
                fileList = [file.name for file in files if '.' in file.name]
            return fileList
        except exceptions.FirebaseError as e:
            if hasattr(e, 'message'):
                LOG.error('<%s> in line %s !!!' % (e.message,format(sys.exc_info()[-1].tb_lineno)))
            else:
                LOG.error('<%s> in line %s !!!' % (e,format(sys.exc_info()[-1].tb_lineno)))
        
        
    def metadata(self,blob_name):
        """Prints out a blob's metadata."""
        # blob_name = 'your-object-name'
        try:
            blob = self.bucket.get_blob(blob_name).metadata
            for n in blob.keys():
                print("{0} {1}".format(n,blob[n]))
        except exceptions.FirebaseError as e:
            if hasattr(e, 'message'):
                LOG.error('<%s> in line %s !!!' % (e.message,format(sys.exc_info()[-1].tb_lineno)))
            else:
                LOG.error('<%s> in line %s !!!' % (e,format(sys.exc_info()[-1].tb_lineno)))

 

class GFirestore(object):
    '''
       @parameters
       cert_file :  json file with firebase cert
       batch     :  operations in batch mode


    '''
    def __init__(self, *args, **kwargs):
        try:

            if kwargs.get('cert_file'):
                json_cert = kwargs['cert_file']
            else:
                json_cert = GOOGLE_CERTIFICATE

            # operaciones por lotes o batch
            if kwargs.get('batch') and kwargs['batch']:
                self.batch = True
                LOG.debug('BATCH TRUE')
            else:
                self.batch = False
                
            cred = credentials.Certificate(json_cert)
            firebase_admin.initialize_app(cred)

            self.db = firestore.client()   # return google.cloud.firestore.Firestore object

        except Exception as e:
            if hasattr(e, 'message'):
                LOG.error('<%s> in line %s !!!' % (e.message,format(sys.exc_info()[-1].tb_lineno)))
            else:
                LOG.error('<%s> in line %s !!!' % (e,format(sys.exc_info()[-1].tb_lineno)))


    def post(self, collection=None, document=dict()):
        self.put(collection=collection, document=document)
        
    def put(self, collection=None, document=dict()):
        """
           @parameters
           collection :  collection name <str>
           document :  <dict> with { 'document_key': { data_dict ...}, }
        """
        try:
            if collection and isinstance(document,(dict,)):

                if self.batch:
                    _batch = self.db.batch()

                for n in document.keys():
                    doc_ref = self.db.collection(u'{0}'.format(collection)).document(u'{0}'.format(n))
                    if self.batch:
                        _batch.set(doc_ref, document[n])
                    else:
                        doc_ref.set(document[n])

                if self.batch:
                    # Commit the batch
                    _batch.commit()
                    # ERROR:firestore:<Quota exceeded.> in line 313 !!!

        except Exception as e:
            if hasattr(e, 'message'):
                LOG.error('<%s> in line %s !!!' % (e.message,format(sys.exc_info()[-1].tb_lineno)))
            else:
                LOG.error('<%s> in line %s !!!' % (e,format(sys.exc_info()[-1].tb_lineno)))
    
    def get(self, collection=None, document=list()):
        """
           @parameters
           collection :  collection name <str>
           document :  <list> with [ 'document_key' , ...]
        """
        try:
            if collection and isinstance(document,(list,tuple,)):
                _documents = dict()
                if len(document)>0:
                    for n in document:
                        doc_ref = self.db.collection(u"""{0}""".format(collection)).document(u"""{0}""".format(n))
                        doc = doc_ref.get()
                        _documents[n] = doc.to_dict()
                else:
                    # obtener objetos por seleccion
                    '''
                    docs = self.db.collection(u'users').where(u'born', u'==', 1815).stream()
                    for doc in docs:
                        print(u'{} => {}'.format(doc.id, doc.to_dict()))
                    '''
                    # todos los documentos de una coleccion
                    docs = self.db.collection(u"""{0}""".format(collection)).stream()
                    for doc in docs:
                        print(u'{} => {}'.format(doc.id, doc.to_dict()))

        except Exception as e:
            if hasattr(e, 'message'):
                LOG.error('<%s> in line %s !!!' % (e.message,format(sys.exc_info()[-1].tb_lineno)))
            else:
                LOG.error('<%s> in line %s !!!' % (e,format(sys.exc_info()[-1].tb_lineno)))


    def delete(self, collection=None, document=list()):
        """
           @parameters
           collection :  collection name <str>
           document :  <list> with [ 'document_key' , ...]
        """
        try:
            if collection and isinstance(document,(list,tuple,)):

                if self.batch:
                    _batch = self.db.batch()
                
                _documents = dict()
                if len(document)>0:
                    for n in document:
                        doc_ref = self.db.collection(u'{0}'.format(collection)).document(u'{0}'.format(n))
                        if self.batch:
                            _batch.delete(doc_ref)
                        else:
                            doc = doc_ref.delete()
                if self.batch:
                    # Commit the batch
                    _batch.commit()
                            
        except Exception as e:
            if hasattr(e, 'message'):
                LOG.error('<%s> in line %s !!!' % (e.message,format(sys.exc_info()[-1].tb_lineno)))
            else:
                LOG.error('<%s> in line %s !!!' % (e,format(sys.exc_info()[-1].tb_lineno)))


AUTHORIZED_KEYS = ['email',
                   'phone_number',
                   'email_verified',
                   'password',
                   'display_name',
                   'photo_url',
                   'disabled',
                   'app']

class GAuth(object):
    def __init__(self, *args, **kwargs):
        try:
            self.bucketName = BUCKET_NAME
            self.bucketFolder = BUCKET_FOLDER
            cred = credentials.Certificate(GOOGLE_CERTIFICATE)
            firebase_admin.initialize_app(cred, {'storageBucket': self.bucketName})

        except exceptions.FirebaseError as e:
            if hasattr(e, 'message'):
                LOG.error('<%s> in line %s !!!' % (e.message,format(sys.exc_info()[-1].tb_lineno)))
            else:
                LOG.error('<%s> in line %s !!!' % (e,format(sys.exc_info()[-1].tb_lineno)))

    def create_user(self,kwargs):
        """Upload files to GCP bucket.
        uid 	string 	El uid que se asignará al usuario recién creado. Debe ser una string que contenga entre 1 y 128 caracteres. Si no se proporciona, se generará automáticamente un uid aleatorio.
        email 	string 	El correo electrónico principal del usuario. Debe ser una dirección de correo electrónico válida.
        emailVerified 	booleano 	Indica si se verificó el correo electrónico principal del usuario. Si no se proporciona, el valor predeterminado es false.
        phoneNumber 	string 	El número de teléfono principal del usuario. Debe ser un número de teléfono válido que cumpla con las especificaciones E.164.
        password 	string 	La contraseña del usuario, sin encriptación. Debe tener al menos seis caracteres.
        displayName 	string 	El nombre visible del usuario.
        photoURL 	string 	La URL de la foto del usuario.
        disabled 	booleano 	Si el usuario está inhabilitado o no. true indica que está inhabilitado; false indica que está habilitado. Si no se proporciona, el valor predeterminado es false. 
        app:            An App instance (optional)
        """
        # uid se crea de forma aleatoria
        try:
            user_auth = dict()
            print(kwargs)
            print(**kwargs)
            for key in kwargs.keys():
                if key in AUTHORIZED_KEYS and kwargs[key] is not None:
                    user_auth[key] = kwargs[key]
            if not user_auth.get('email_verified',False):
                user_auth['email_verified'] = False
            if not user_auth.get('disabled',False):
                user_auth['disabled'] = False
            print(**user_auth)

            ''' si deseo crear mi usuario con uid propio 
            
            user = auth.create_user(
                uid='some-uid', email='user@example.com', phone_number='+15555550100')
            print('Sucessfully created new user: {0}'.format(user.uid))
            '''

        except exceptions.FirebaseError as e:
            if hasattr(e, 'message'):
                LOG.error('<%s> in line %s !!!' % (e.message,format(sys.exc_info()[-1].tb_lineno)))
            else:
                LOG.error('<%s> in line %s !!!' % (e,format(sys.exc_info()[-1].tb_lineno)))

    def verify_email(self,email):
        """email verify
           @parameter
               email
           @return
               The email verification link created by the API. Send email to user with link
        """
        try:
            email_verify_link = auth.generate_email_verification_link(email, action_code_settings=None, app=None)
            return(email_verify_link)
        except exceptions.FirebaseError as e:
            if hasattr(e, 'message'):
                LOG.error('<%s> in line %s !!!' % (e.message,format(sys.exc_info()[-1].tb_lineno)))
            else:
                LOG.error('<%s> in line %s !!!' % (e,format(sys.exc_info()[-1].tb_lineno)))

    def reset_password(self,email):
        """password reset
           @parameter
               email
           @return
               The email verification link created by the API. Send email to user with link
        """
        try:
            reset_password_link = auth.generate_password_reset_link(email, action_code_settings=None, app=None)
            return(reset_password_link)
        except exceptions.FirebaseError as e:
            if hasattr(e, 'message'):
                LOG.error('<%s> in line %s !!!' % (e.message,format(sys.exc_info()[-1].tb_lineno)))
            else:
                LOG.error('<%s> in line %s !!!' % (e,format(sys.exc_info()[-1].tb_lineno)))

    def update_user(self):
        try:
            user = auth.update_user(
                uid,
                email='user@example.com',
                phone_number='+15555550100',
                email_verified=True,
                password='newPassword',
                display_name='John Doe',
                photo_url='http://www.example.com/12345678/photo.png',
                disabled=True)
        except exceptions.ValueError as e:
            if hasattr(e, 'message'):
                LOG.error('<%s> in line %s !!!' % (e.message,format(sys.exc_info()[-1].tb_lineno)))
            else:
                LOG.error('<%s> in line %s !!!' % (e,format(sys.exc_info()[-1].tb_lineno)))
        except exceptions.FirebaseError as e:
            if hasattr(e, 'message'):
                LOG.error('<%s> in line %s !!!' % (e.message,format(sys.exc_info()[-1].tb_lineno)))
            else:
                LOG.error('<%s> in line %s !!!' % (e,format(sys.exc_info()[-1].tb_lineno)))

    def delete_user(self, uid):
        try:        
            auth.delete_user(uid, app=None)
        except exceptions.FirebaseError as e:
            if hasattr(e, 'message'):
                LOG.error('<%s> in line %s !!!' % (e.message,format(sys.exc_info()[-1].tb_lineno)))
            else:
                LOG.error('<%s> in line %s !!!' % (e,format(sys.exc_info()[-1].tb_lineno)))

    def get_user(self,**kwargs):
        """
           @parameters
               uid || email || phone
           @return
               UserRecord
        """
        try:
            user_record = None
            if kwargs.get('uid',None):
                # Gets the user data corresponding to the specified user ID
                user_record = auth.get_user(kwargs['uid'])
            elif kwargs.get('email',None):
                # Gets the user data corresponding to the specified user email
                user_record = auth.get_user_by_email(kwargs['email'])
            elif kwargs.get('phone',None):
                # Gets the user data corresponding to the specified phone number.
                user_record = auth.get_user_by_phone_number(kwargs['phone'])
            print(user_record.to_dict())
            print(user_record.uid)
            print(user_record.email_verified)
            return user_record.uid
        except exceptions.FirebaseError as e:
            if hasattr(e, 'message'):
                LOG.error('<%s> in line %s !!!' % (e.message,format(sys.exc_info()[-1].tb_lineno)))
            else:
                LOG.error('<%s> in line %s !!!' % (e,format(sys.exc_info()[-1].tb_lineno)))

    def user_list(self):
        # Start listing users from the beginning, 1000 at a time.
        try:
            """
            page_token – A non-empty page token string, which indicates the starting point of the page (optional). Defaults to None, which will retrieve the first page of users.
            max_results – A positive integer indicating the maximum number of users to include in the returned page (optional). Defaults to 1000, which is also the maximum number allowed.
            app – An App instance (optional).
            """
            page = auth.list_users(page_token=None, max_results=1000, app=None)
            while page:
                for user in page.users:
                    print('User: ' + user.uid)
                # Get next batch of users.
                page = page.get_next_page()

            # Iterate through all users. This will still retrieve users in batches,
            # buffering no more than 1000 users in memory at a time.
            """
            for user in auth.list_users().iterate_all():
                print('User: ' + user.uid)
            """
        except exceptions.FirebaseError as e:
            if hasattr(e, 'message'):
                LOG.error('<%s> in line %s !!!' % (e.message,format(sys.exc_info()[-1].tb_lineno)))
            else:
                LOG.error('<%s> in line %s !!!' % (e,format(sys.exc_info()[-1].tb_lineno)))



if __name__ == "__main__":

    print ('''copyright {0}'''.format( __copyright__))
    print ('''license {0}'''.format( __license__))
    print ('''version {0}'''.format( __version__))
    if len(sys.argv) < 2:
        sys.stderr.write("For help use -h o --help")
    elif sys.argv[1]=='-h' or sys.argv[1]=='--help':
        print (''' Google Firestore data ''')



#!/usr/bin/python
# -*- coding: utf-8 -*-

# This file is part of Pymetrick.  The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.

"""Modulo para identificar clientes conectados"""

try:
    from pymetrick.version import __author__, __version__, __license__, __copyright__
except ImportError:
    raise ImportError(
        'Failed to find (autogenerated) version.py '
    )

__date__ = '2012-09-21'
__modify__ = '2019-08-26'
__credits__ = ''
__text__ = 'Identificacion de conexiones y funciones de encriptacion'
__file__ = 'session.py'

#--- CHANGES ------------------------------------------------------------------
# 2012-09-21 v0.01 PL: - First version
# 2014-08-30 v0.02 PL: - Second version
# 2015-09-10 v0.03 PL: - Adaptado a python 3.x
# 2017-10-02 v0.43 PL: - Migrar a PYTHON 3.6
# 2018-02-13 v0.44 PL: - Add json web tokens JWT
# 2019-08-26 v0.49.0 PL: - Add encryp and decrypt with AES

import sys
import os
import string
import stat
import hmac
import binascii
import hashlib
import pyaes
import logging
import logging.handlers
import copy
import jwt

try:
    import json
except:
    import simplejson as json

try:
    from itertools import izip as zip
except ImportError: # will be 3.x series
    pass

from datetime import date, time, datetime, timedelta
from random import SystemRandom,choice
from time import localtime,mktime,sleep
from pymetrick.helpers import *
from pymetrick import ipaddress



# Caracteres permitidos en session
ALLOWED_CHARS = '''abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789'''
# Para ofuscar la encriptacion de password
SALT_CHARS = '''abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789'''
# Caracteres desordenados para generar una password aleatoria
PASSWORD_CHARS = '''BCIOQSVXYZclovxz23478ADEFGHJKLMNPRTUW15690abdefghijkmnpqrstuwy'''
# Numero de veces que se encriptara la password
DEFAULT_PBKDF2_ITERATIONS = 1000
long = int

'''
DEBUG - debug message
INFO - info message
WARNING - warn message
ERROR - error message
CRITICAL - critical message
'''
LOG_LEVEL_VALUE = {'DEBUG':10,'INFO':20,'WARNING':30,'ERROR':40,'CRITICAL':50}
if str(os.environ.get('PYMETRICK_LOG_LEVEL',None)).upper() in LOG_LEVEL_VALUE:
    LOG_LEVEL = LOG_LEVEL_VALUE[str(os.environ.get('PYMETRICK_LOG_LEVEL')).upper()]
else:
    LOG_LEVEL = 30
LOG_FILENAME = '-'.join([os.path.abspath(__file__).split(os.sep)[len(os.path.abspath(__file__).split(os.sep))-1],])[:-3]
LOG = logging.getLogger(LOG_FILENAME)

if 'LD_LIBRARY_PATH' in list(os.environ.keys()):
    # CGI environment
    sys.stdout = sys.stderr
    logging.basicConfig(stream = sys.stderr, level=LOG_LEVEL, format='%(filename)s[line:%(lineno)d] - %(levelname)s - %(message)s')
else:
    # not CGI environment
    logging.basicConfig(stream=sys.stderr)
    hdlr = logging.handlers.RotatingFileHandler(filename=LOG_FILENAME+'.log',mode='a', encoding='utf-8', maxBytes=1048576, backupCount=3)
    formatter = logging.Formatter('%(asctime)s - %(filename)s[line:%(lineno)d] - %(levelname)s - %(message)s')
    hdlr.setFormatter(formatter)
    LOG.addHandler(hdlr)
    LOG.setLevel(LOG_LEVEL)

_builtin_safe_str_cmp = getattr(hmac, 'compare_digest', None)
_sys_rng = SystemRandom()

AUTH_HEADER_NAME = 'Authorization'
JWT_ENCODING_ALGORITHM = 'HS512'
JWT_SECRET_KEY = b'A0B9C8D7E6Z1Y2X3W4V5'
JWT_DEFAULT_LIFETIME = 60 * 60 * 24 * 7     # seconds
#JWT_DEFAULT_LIFETIME = 5     # seconds

class JWTLogin:
    ''' JSON WEB TOKENS stateless authentication '''

    def __init__(self, secret=None, lifetime=None):
        '''
           @Parameters
           secret -> string with secret key
           lifetime -> seconds to expired auth
        '''
        try:
            if not secret:
                self.secret_key = JWT_SECRET_KEY
            else:
                self.secret_key = secret

            if not lifetime:
                self.lifetime = JWT_DEFAULT_LIFETIME
            else:
                self.lifetime = lifetime

            self.encode(payload={'sub':None,'role':None})

        except Exception as e:
            if hasattr(e, 'message'):
                LOG.error('JWTLogin <%s> in line %s !!!' % (e.message,format(sys.exc_info()[-1].tb_lineno)))
            else:
                LOG.error('JWTLogin <%s> in line %s !!!' % (e,format(sys.exc_info()[-1].tb_lineno)))


    def encode(self, payload=dict()):
        '''
        @Parameters
           payload : dict
           lifetima : seconds
        '''

        try:
            payload['iat'] = datetime.utcnow()
            payload['exp'] = payload['iat'] + timedelta(seconds=self.lifetime)
            return jwt.encode(payload, self.secret_key, algorithm= JWT_ENCODING_ALGORITHM)

        except Exception as e:
            if hasattr(e, 'message'):
                LOG.error('JWTLogin.encode <%s> in line %s !!!' % (e.message,format(sys.exc_info()[-1].tb_lineno)))
            else:
                LOG.error('JWTLogin.encode <%s> in line %s !!!' % (e,format(sys.exc_info()[-1].tb_lineno)))

    def decode(self, token=''):
        try:
            return jwt.decode(token, self.secret_key, algorithms=[JWT_ENCODING_ALGORITHM])
        except jwt.ExpiredSignatureError:
            # Signature has expired
            return False        
        except Exception as e:
            if hasattr(e, 'message'):
                LOG.error('JWTLogin.encode <%s> in line %s !!!' % (e.message,format(sys.exc_info()[-1].tb_lineno)))
            else:
                LOG.error('JWTLogin.encode <%s> in line %s !!!' % (e,format(sys.exc_info()[-1].tb_lineno)))
            return False


def safe_str_cmp(a, b):
    '''This function compares strings in somewhat constant time.  This
    requires that the length of at least one string is known in advance.
    Returns `True` if the two strings are equal or `False` if they are not.
    '''
    try:
        if _builtin_safe_str_cmp is not None:
            return _builtin_safe_str_cmp(a, b)
        if len(a) != len(b):
            return False
        rv = 0
        for x, y in zip(a, b):
            rv |= ord(x) ^ ord(y)
        return rv == 0
    except Exception as e:
        if hasattr(e, 'message'):
            LOG.error('safe_str_cmp <%s> in line %s !!!' % (e.message,format(sys.exc_info()[-1].tb_lineno)))
        else:
            LOG.error('safe_str_cmp <%s> in line %s !!!' % (e,format(sys.exc_info()[-1].tb_lineno)))


def gen_salt(length = 8):
    '''Genera una cadena aleatoria de SALT_CHARS
       @Parameters
           length : longitud de cadena de caracteres a obtener
       @Return
            una cadena de caracteres aleatoria

    '''
    try:
        if length <= 0:
            raise ValueError('requested salt of length <= 0')
        return (''.join(_sys_rng.choice(SALT_CHARS) for _ in range(length))).strip()
    except Exception as e:
        if hasattr(e, 'message'):
            LOG.error('gen_salt <%s> in line %s !!!' % (e.message,format(sys.exc_info()[-1].tb_lineno)))
        else:
            LOG.error('gen_salt <%s> in line %s !!!' % (e,format(sys.exc_info()[-1].tb_lineno)))

def _find_hashlib_algorithms():
    try:
        algos = getattr(hashlib, 'algorithms', None)
        if algos is None:
            algos = ('md5', 'sha1', 'sha224', 'sha256', 'sha384', 'sha512')
        # mostrar metodos disponibles
        rv = {}
        for algo in algos:
            func = getattr(hashlib, algo, None)
            if func is not None:
                rv[algo] = func
        return rv
    except Exception as e:
        if hasattr(e, 'message'):
            LOG.error('_find_hashlib_algorithms <%s> in line %s !!!' % (e.message,format(sys.exc_info()[-1].tb_lineno)))
        else:
            LOG.error('_find_hashlib_algorithms <%s> in line %s !!!' % (e,format(sys.exc_info()[-1].tb_lineno)))

_hash_funcs = _find_hashlib_algorithms()

def pbkdf2_hex(data, salt, iterations=DEFAULT_PBKDF2_ITERATIONS,
               keylen=None, hashfunc=None):
    '''como la funcion 'pbkdf2_bin' pero devuelve una cadena hexadecimal.

    @Parameters
        data: the data to derive.
        salt: the salt for the derivation.
        iterations: the number of iterations.
        keylen: the length of the resulting key.  If not provided
                   the digest size will be used.
        hashfunc: the hash function to use.  This can either be the
                     string name of a known hash function or a function
                     from the hashlib module.  Defaults to sha1.
    '''
    try:
        rv = pbkdf2_bin(data, salt, iterations, keylen, hashfunc)
        return to_native(codecs.encode(rv, 'hex_codec'))
    except Exception as e:
        if hasattr(e, 'message'):
            LOG.error('pbkdf2_hex <%s> in line %s !!!' % (e.message,format(sys.exc_info()[-1].tb_lineno)))
        else:
            LOG.error('pbkdf2_hex <%s> in line %s !!!' % (e,format(sys.exc_info()[-1].tb_lineno)))


def pbkdf2_bin(data, salt, iterations=DEFAULT_PBKDF2_ITERATIONS,
               keylen=None, hashfunc=None):
    '''Returns a binary digest for the PBKDF2 hash algorithm of `data`
    with the given `salt`. It iterates `iterations` time and produces a
    key of `keylen` bytes. By default SHA-1 is used as hash function,
    a different hashlib `hashfunc` can be provided.

    @Parameters
        data: the data to derive.
        salt: the salt for the derivation.
        iterations: the number of iterations.
        keylen: the length of the resulting key.  If not provided
                   the digest size will be used.
        hashfunc: the hash function to use.  This can either be the
                     string name of a known hash function or a function
                     from the hashlib module.  Defaults to sha1.
    '''
    try:
        if isinstance(hashfunc, string_types):
            hashfunc = _hash_funcs[hashfunc]
        elif not hashfunc:
            hashfunc = hashlib.sha1
        salt = to_bytes(salt)
        mac = hmac.HMAC(to_bytes(data), None, hashfunc)
        if not keylen:
            keylen = mac.digest_size
        def _pseudorandom(x, mac=mac):
            h = copy.copy(mac)
            h.update(x)
            return bytearray(h.digest())
        buf = bytearray()
        for block in range_type(1, -(-keylen // mac.digest_size) + 1):
            rv = u = _pseudorandom(salt + _pack_int(block))
            for i in range_type(iterations - 1):
                u = _pseudorandom(bytes(u))
                rv = bytearray(starmap(xor, izip(rv, u)))
            buf.extend(rv)
        return bytes(buf[:keylen])
    except Exception as e:
        if hasattr(e, 'message'):
            LOG.error('pbkdf2_bin <%s> in line %s !!!' % (e.message,format(sys.exc_info()[-1].tb_lineno)))
        else:
            LOG.error('pbkdf2_bin <%s> in line %s !!!' % (e,format(sys.exc_info()[-1].tb_lineno)))

def _hash_internal(method, salt , password):
    '''Contraseña Interna ayudante hash. Soporta texto claro sin ofuscar,
       contraseñas sin ofuscar y ofuscada. En el caso de contraseñas ofuscadas se utiliza
       HMAC.
    @Parameters
        method : metodo para encriptar la password, permite devolver el texto sin encriptar con 'plain'
        salt   : cadema de caracteres que permitira ofuscar los caracteres que componen la password
        password : texto que deseamos tratar.
    @Return
        tuple( texto de password encriptado y/o ofuscado, metodo de encriptacion)
    '''
    try:
        if method == 'plain':
            return password, method

        password = password.encode('utf-8')

        if method.startswith('pbkdf2:'):
            args = method[7:].split(':')
            if len(args) not in (1, 2):
                raise ValueError('Invalid number of arguments for PBKDF2')
            method = args.pop(0)
            iterations = args and int(args[0] or 0) or DEFAULT_PBKDF2_ITERATIONS
            is_pbkdf2 = True
            actual_method = 'pbkdf2:%s:%d' % (method, iterations)
        else:
            is_pbkdf2 = False
            actual_method = method

        hash_func = _hash_funcs.get(method)
        if hash_func is None:
            raise TypeError('invalid method %r' % method)

        if is_pbkdf2:
            if not salt:
                raise ValueError('Salt is required for PBKDF2')
            rv = pbkdf2_hex(password, salt, iterations,
                            hashfunc=hash_func)
        elif salt:
            salt = salt.encode('utf-8')
            rv = hmac.HMAC(salt, password, hash_func).hexdigest()
        else:
            h = hash_func()
            h.update(password)
            rv = h.hexdigest()
        return rv, actual_method
    except Exception as e:
        if hasattr(e, 'message'):
            LOG.error('_hash_internal <%s> in line %s !!!' % (e.message,format(sys.exc_info()[-1].tb_lineno)))
        else:
            LOG.error('_hash_internal <%s> in line %s !!!' % (e,format(sys.exc_info()[-1].tb_lineno)))

def generate_password_hash(password, method='sha1', salt_length=8):
    '''Encripta una contraseña con el método dado y derivada de ofuscacion con la longitud
    dada. El formato de la cadena devuelta incluye el método que se utilizó de forma que
    `check_password_hash` pueda comprobar el hash.

    El formato de la cadena hash se ve así::

        method$salt$hash

    Este método no puede generar contraseñas sin ofuscacion, pero es posible
    generar las contraseñas en texto plano. Si utiliza derivada de ofuscacion,
    HMAC se utiliza internamente para ofuscar la contraseña.

    @parameters
        password: cadena de caracteres a encriptar
        method: metodo de encriptacion empleado (soportado por hashlib)
        salt_length: numero de caracteres usados de ofuscar la contraseña
    '''
    try:
        salt = method != 'plain' and gen_salt(salt_length) or ''
        h, actual_method = _hash_internal(method, salt, password)
        return '%s$%s$%s' % (actual_method, salt, h)
    except Exception as e:
        if hasattr(e, 'message'):
            LOG.error('generate_password_hash <%s> in line %s !!!' % (e.message,format(sys.exc_info()[-1].tb_lineno)))
        else:
            LOG.error('generate_password_hash <%s> in line %s !!!' % (e,format(sys.exc_info()[-1].tb_lineno)))


def check_password_hash(pwhash, password):
    '''comprobar una contraseña con un valor de contraseña dada salada y hash.
    Con el fin de apoyar las contraseñas sin sal heredados Este método admite
    contraseñas de texto sin formato MD5 y SHA1 hashes (tanto con y sin sal).

    Returns `True` if the password matched, `False` otherwise.
    @Parameters
        pwhash: una password encriptada como la que se genera desde generate_password_hash()
        password: texto sin encriptar, correspondiente a la password, para comparar.
    @Return
        'True' si la password coincide o 'False' en cualquier otro caso.
    '''
    try:
        if pwhash.count('$') < 2:
            return False
        method, salt, hashval = pwhash.split('$', 2)
        return safe_str_cmp(_hash_internal(method, salt, password)[0], hashval)
    except Exception as e:
        if hasattr(e, 'message'):
            LOG.error('check_password_hash <%s> in line %s !!!' % (e.message,format(sys.exc_info()[-1].tb_lineno)))
        else:
            LOG.error('check_password_hash <%s> in line %s !!!' % (e,format(sys.exc_info()[-1].tb_lineno)))

def csrf_token(csrf_length=32):
    '''genera un string con valores aleatorios con una longitud determinada
       orientado al Cross Site Request Forgery protection
       puede ser usado con las comunicaciones ajax
       en la sesion del servidor deberá indicarse como csrftoken y se indicara
       el valor aleatorio en cada pagina generada, recibiendo el mismo valor
       cuando se realice una solicitud ajax o envío de un formulario GET, POST,
       PUT, DELETE.
       @Parameters
           csrf_length : longitud de la cadena de caracteres a obtener
       @Return
           cadena de caracteres aleatoria con la longitud solicitada
    '''
    try:
        return (gen_salt(csrf_length))
    except Exception as e:
        if hasattr(e, 'message'):
            LOG.error('csrf_token <%s> in line %s !!!' % (e.message,format(sys.exc_info()[-1].tb_lineno)))
        else:
            LOG.error('csrf_token <%s> in line %s !!!' % (e,format(sys.exc_info()[-1].tb_lineno)))

def new_password(length = 15, chars = None):
    '''Genera password aleatoria dependiendo de la cadena de caracteres recibida o cadena por defecto
       @Parameters
                length = longitud de password
                chars  = cadena de caracteres para generacion aleatoria de password, aunque por defecto
                         puede utilizar una cadena propia
       @Return
                String = password sin encriptacion de la longitud solicitada u 8 caracteres por defecto
    '''
    try:
        local_chars = PASSWORD_CHARS if(chars) is None else chars;
        if len(local_chars)<30:
            # si la cadena recibida contiene menos de 30 caracteres se amplia de letras y numeros
            local_chars = string.letters + string.digits
        return ''.join(choice(local_chars) for _ in range(length))
    except Exception as e:
        if hasattr(e, 'message'):
            LOG.error('new_password <%s> in line %s !!!' % (e.message,format(sys.exc_info()[-1].tb_lineno)))
        else:
            LOG.error('new_password <%s> in line %s !!!' % (e,format(sys.exc_info()[-1].tb_lineno)))

def file2checksums(__path__, blocksize=65536):
    '''obtener el hash de un fichero para identificarlo de forma unica
    '''
    try:
        hash = hashlib.md5()
        with open(__path__, "rb", encoding='utf-8') as f:
            block_read = ( lambda: f.read( blocksize ))
            for block in iter( block_read, b'' ):
                hash.update( block )
        return hash.hexdigest()
    except Exception as e:
        if hasattr(e, 'message'):
            LOG.error('file2checksums <%s> in line %s !!!' % (e.message,format(sys.exc_info()[-1].tb_lineno)))
        else:
            LOG.error('file2checksums <%s> in line %s !!!' % (e,format(sys.exc_info()[-1].tb_lineno)))

# AES encrypt or decrypt
def encryptAES(data,key=None):
    if key is None or key.length()<33:
        # 256 bit keys
        key = PASSWORD_CHARS[0:32]
    key = str2bytes(key,'utf-8')
    aes = pyaes.AESModeOfOperationCTR(key)    
    ciphertext = aes.encrypt(data)
    return ciphertext

def decryptAES(data,key=None):
    if key is None or key.length()<33:
        # 256 bit keys
        key = PASSWORD_CHARS[0:32]
    key = key.encode('utf-8')
    aes = pyaes.AESModeOfOperationCTR(key)    
    plaintext = bytes2str(aes.decrypt(data),'utf-8')
    return plaintext

if __name__ == "__main__":
    print ('''copyright {0}'''.format( __copyright__))
    print ('''license {0}'''.format( __license__))
    print ('''version {0}'''.format( __version__))
    if len(sys.argv) < 2:
        sys.stderr.write("For help use -h o --help")
    elif sys.argv[1]=='-h' or sys.argv[1]=='--help':
        print ('''Gestion de seguridad de acceso ''')

    
    

